@@ -14,9 +14,8 @@ Documentation is available at https://pytmd.readthedocs.io
 import pyTMD.eop
 import pyTMD.time
 import pyTMD.utilities
+import pyTMD.version
 from pyTMD.bilinear_interp import bilinear_interp
-from pyTMD.convert_julian import convert_julian
-from pyTMD.convert_calendar_decimal import convert_calendar_decimal
 from pyTMD.calc_delta_time import calc_delta_time
 from pyTMD.calc_astrol_longitudes import calc_astrol_longitudes
 from pyTMD.compute_equilibrium_tide import compute_equilibrium_tide
@@ -26,6 +25,7 @@ from pyTMD.iers_mean_pole import iers_mean_pole
 from pyTMD.infer_minor_corrections import infer_minor_corrections
 from pyTMD.load_constituent import load_constituent
 from pyTMD.load_nodal_corrections import load_nodal_corrections
+from pyTMD.nearest_extrap import nearest_extrap
 from pyTMD.output_otis_tides import output_otis_grid
 from pyTMD.read_tide_model import extract_tidal_constants
 from pyTMD.read_netcdf_model import extract_netcdf_constants
@@ -37,3 +37,6 @@ from pyTMD.predict_tide import predict_tide
 from pyTMD.read_iers_EOP import read_iers_EOP
 from pyTMD.read_ocean_pole_tide import read_ocean_pole_tide
 from pyTMD.tidal_ellipse import tidal_ellipse
+
+# get semantic version from setuptools-scm
+__version__ = pyTMD.version.version
