@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 u"""
 compute_tide_corrections.py
-Written by Tyler Sutterley (11/2020)
+Written by Tyler Sutterley (03/2021)
 Calculates tidal elevations for correcting elevation or imagery data
 
 Uses OTIS format tidal solutions provided by Ohio State University and ESR
@@ -34,6 +34,7 @@ OPTIONS:
         bilinear: quick bilinear interpolation
         spline: scipy bivariate spline interpolation
         linear, nearest: scipy regular grid interpolations
+    EXTRAPOLATE: extrapolate with nearest-neighbors
     FILL_VALUE: output invalid value (default NaN)
 
 PYTHON DEPENDENCIES:
@@ -62,8 +63,14 @@ PROGRAM DEPENDENCIES:
     read_netcdf_model.py: extract tidal harmonic constants from netcdf models
     read_GOT_model.py: extract tidal harmonic constants from GSFC GOT models
     read_FES_model.py: extract tidal harmonic constants from FES tide models
+    bilinear_interp.py: bilinear interpolation of data to coordinates
+    nearest_extrap.py: nearest-neighbor extrapolation of data to coordinates
 
 UPDATE HISTORY:
+    Updated 03/2021: added TPXO9-atlas-v4 in binary OTIS format
+        simplified netcdf inputs to be similar to binary OTIS read program
+    Updated 02/2021: replaced numpy bool to prevent deprecation warning
+    Updated 12/2020: added valid data extrapolation with nearest_extrap
     Updated 11/2020: added model constituents from TPXO9-atlas-v3
     Updated 08/2020: using builtin time operations.
         calculate difference in leap seconds from start of epoch
@@ -80,6 +87,7 @@ import pyproj
 import datetime
 import numpy as np
 import pyTMD.time
+import pyTMD.utilities
 from pyTMD.calc_delta_time import calc_delta_time
 from pyTMD.infer_minor_corrections import infer_minor_corrections
 from pyTMD.predict_tide import predict_tide
@@ -92,7 +100,7 @@ from pyTMD.read_FES_model import extract_FES_constants
 #-- PURPOSE: compute tides at points and times using tide model algorithms
 def compute_tide_corrections(x, y, delta_time, DIRECTORY=None, MODEL=None,
     EPSG=3031, EPOCH=(2000,1,1,0,0,0), TYPE='drift', TIME='UTC',
-    METHOD='spline', FILL_VALUE=np.nan):
+    METHOD='spline', EXTRAPOLATE=False, FILL_VALUE=np.nan):
     """
     Compute tides at points and times using tidal harmonics
 
@@ -121,6 +129,7 @@ def compute_tide_corrections(x, y, delta_time, DIRECTORY=None, MODEL=None,
         bilinear: quick bilinear interpolation
         spline: scipy bivariate spline interpolation
         linear, nearest: scipy regular grid interpolations
+    EXTRAPOLATE: extrapolate with nearest-neighbors
     FILL_VALUE: output invalid value (default NaN)
 
     Returns
@@ -149,31 +158,35 @@ def compute_tide_corrections(x, y, delta_time, DIRECTORY=None, MODEL=None,
         model_type = 'z'
     elif (MODEL == 'TPXO9-atlas'):
         model_directory = os.path.join(DIRECTORY,'TPXO9_atlas')
-        grid_file = 'grid_tpxo9_atlas.nc.gz'
+        grid_file = os.path.join(model_directory,'grid_tpxo9_atlas.nc.gz')
         model_files = ['h_q1_tpxo9_atlas_30.nc.gz','h_o1_tpxo9_atlas_30.nc.gz',
             'h_p1_tpxo9_atlas_30.nc.gz','h_k1_tpxo9_atlas_30.nc.gz',
             'h_n2_tpxo9_atlas_30.nc.gz','h_m2_tpxo9_atlas_30.nc.gz',
             'h_s2_tpxo9_atlas_30.nc.gz','h_k2_tpxo9_atlas_30.nc.gz',
             'h_m4_tpxo9_atlas_30.nc.gz','h_ms4_tpxo9_atlas_30.nc.gz',
             'h_mn4_tpxo9_atlas_30.nc.gz','h_2n2_tpxo9_atlas_30.nc.gz']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         model_format = 'netcdf'
         model_type = 'z'
         SCALE = 1.0/1000.0
+        GZIP = True
     elif (MODEL == 'TPXO9-atlas-v2'):
         model_directory = os.path.join(DIRECTORY,'TPXO9_atlas_v2')
-        grid_file = 'grid_tpxo9_atlas_30_v2.nc.gz'
+        grid_file = os.path.join(model_directory,'grid_tpxo9_atlas_30_v2.nc.gz')
         model_files = ['h_q1_tpxo9_atlas_30_v2.nc.gz','h_o1_tpxo9_atlas_30_v2.nc.gz',
             'h_p1_tpxo9_atlas_30_v2.nc.gz','h_k1_tpxo9_atlas_30_v2.nc.gz',
             'h_n2_tpxo9_atlas_30_v2.nc.gz','h_m2_tpxo9_atlas_30_v2.nc.gz',
             'h_s2_tpxo9_atlas_30_v2.nc.gz','h_k2_tpxo9_atlas_30_v2.nc.gz',
             'h_m4_tpxo9_atlas_30_v2.nc.gz','h_ms4_tpxo9_atlas_30_v2.nc.gz',
             'h_mn4_tpxo9_atlas_30_v2.nc.gz','h_2n2_tpxo9_atlas_30_v2.nc.gz']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         model_format = 'netcdf'
         model_type = 'z'
         SCALE = 1.0/1000.0
+        GZIP = True
     elif (MODEL == 'TPXO9-atlas-v3'):
         model_directory = os.path.join(DIRECTORY,'TPXO9_atlas_v3')
-        grid_file = 'grid_tpxo9_atlas_30_v3.nc.gz'
+        grid_file = os.path.join(model_directory,'grid_tpxo9_atlas_30_v3.nc.gz')
         model_files = ['h_q1_tpxo9_atlas_30_v3.nc.gz','h_o1_tpxo9_atlas_30_v3.nc.gz',
             'h_p1_tpxo9_atlas_30_v3.nc.gz','h_k1_tpxo9_atlas_30_v3.nc.gz',
             'h_n2_tpxo9_atlas_30_v3.nc.gz','h_m2_tpxo9_atlas_30_v3.nc.gz',
@@ -181,9 +194,25 @@ def compute_tide_corrections(x, y, delta_time, DIRECTORY=None, MODEL=None,
             'h_m4_tpxo9_atlas_30_v3.nc.gz','h_ms4_tpxo9_atlas_30_v3.nc.gz',
             'h_mn4_tpxo9_atlas_30_v3.nc.gz','h_2n2_tpxo9_atlas_30_v3.nc.gz',
             'h_mf_tpxo9_atlas_30_v3.nc.gz','h_mm_tpxo9_atlas_30_v3.nc.gz']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         model_format = 'netcdf'
-        TYPE = 'z'
+        model_type = 'z'
         SCALE = 1.0/1000.0
+        GZIP = True
+    elif (MODEL == 'TPXO9-atlas-v4'):
+        model_directory = os.path.join(DIRECTORY,'TPXO9_atlas_v4')
+        grid_file = os.path.join(model_directory,'grid_tpxo9_atlas_30_v4')
+        model_files = ['h_q1_tpxo9_atlas_30_v4','h_o1_tpxo9_atlas_30_v4',
+            'h_p1_tpxo9_atlas_30_v4','h_k1_tpxo9_atlas_30_v4',
+            'h_n2_tpxo9_atlas_30_v4','h_m2_tpxo9_atlas_30_v4',
+            'h_s2_tpxo9_atlas_30_v4','h_k2_tpxo9_atlas_30_v4',
+            'h_m4_tpxo9_atlas_30_v4','h_ms4_tpxo9_atlas_30_v4',
+            'h_mn4_tpxo9_atlas_30_v4','h_2n2_tpxo9_atlas_30_v4',
+            'h_mf_tpxo9_atlas_30_v4','h_mm_tpxo9_atlas_30_v4']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
+        model_format = 'OTIS'
+        model_EPSG = '4326'
+        model_type = 'z'
     elif (MODEL == 'TPXO9.1'):
         grid_file = os.path.join(DIRECTORY,'TPXO9.1','DATA','grid_tpxo9')
         model_file = os.path.join(DIRECTORY,'TPXO9.1','DATA','h_tpxo9.v1')
@@ -230,47 +259,53 @@ def compute_tide_corrections(x, y, delta_time, DIRECTORY=None, MODEL=None,
         model_directory = os.path.join(DIRECTORY,'GOT4.7','grids_oceantide')
         model_files = ['q1.d.gz','o1.d.gz','p1.d.gz','k1.d.gz','n2.d.gz',
             'm2.d.gz','s2.d.gz','k2.d.gz','s1.d.gz','m4.d.gz']
-        c = ['q1','o1','p1','k1','n2','m2','s2','k2','s1','m4']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         model_format = 'GOT'
         SCALE = 1.0/100.0
+        GZIP = True
     elif (MODEL == 'GOT4.7_load'):
         model_directory = os.path.join(DIRECTORY,'GOT4.7','grids_loadtide')
         model_files = ['q1load.d.gz','o1load.d.gz','p1load.d.gz','k1load.d.gz',
             'n2load.d.gz','m2load.d.gz','s2load.d.gz','k2load.d.gz',
             's1load.d.gz','m4load.d.gz']
-        c = ['q1','o1','p1','k1','n2','m2','s2','k2','s1','m4']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         model_format = 'GOT'
         SCALE = 1.0/1000.0
+        GZIP = True
     elif (MODEL == 'GOT4.8'):
         model_directory = os.path.join(DIRECTORY,'got4.8','grids_oceantide')
         model_files = ['q1.d.gz','o1.d.gz','p1.d.gz','k1.d.gz','n2.d.gz',
             'm2.d.gz','s2.d.gz','k2.d.gz','s1.d.gz','m4.d.gz']
-        c = ['q1','o1','p1','k1','n2','m2','s2','k2','s1','m4']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         model_format = 'GOT'
         SCALE = 1.0/100.0
+        GZIP = True
     elif (MODEL == 'GOT4.8_load'):
         model_directory = os.path.join(DIRECTORY,'got4.8','grids_loadtide')
         model_files = ['q1load.d.gz','o1load.d.gz','p1load.d.gz','k1load.d.gz',
             'n2load.d.gz','m2load.d.gz','s2load.d.gz','k2load.d.gz',
             's1load.d.gz','m4load.d.gz']
-        c = ['q1','o1','p1','k1','n2','m2','s2','k2','s1','m4']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         model_format = 'GOT'
         SCALE = 1.0/1000.0
+        GZIP = True
     elif (MODEL == 'GOT4.10'):
         model_directory = os.path.join(DIRECTORY,'GOT4.10c','grids_oceantide')
         model_files = ['q1.d.gz','o1.d.gz','p1.d.gz','k1.d.gz','n2.d.gz',
             'm2.d.gz','s2.d.gz','k2.d.gz','s1.d.gz','m4.d.gz']
-        c = ['q1','o1','p1','k1','n2','m2','s2','k2','s1','m4']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         model_format = 'GOT'
         SCALE = 1.0/100.0
+        GZIP = True
     elif (MODEL == 'GOT4.10_load'):
         model_directory = os.path.join(DIRECTORY,'GOT4.10c','grids_loadtide')
         model_files = ['q1load.d.gz','o1load.d.gz','p1load.d.gz','k1load.d.gz',
             'n2load.d.gz','m2load.d.gz','s2load.d.gz','k2load.d.gz',
             's1load.d.gz','m4load.d.gz']
-        c = ['q1','o1','p1','k1','n2','m2','s2','k2','s1','m4']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         model_format = 'GOT'
         SCALE = 1.0/1000.0
+        GZIP = True
     elif (MODEL == 'FES2014'):
         model_directory = os.path.join(DIRECTORY,'fes2014','ocean_tide')
         model_files = ['2n2.nc.gz','eps2.nc.gz','j1.nc.gz','k1.nc.gz',
@@ -280,12 +315,14 @@ def compute_tide_corrections(x, y, delta_time, DIRECTORY=None, MODEL=None,
             'mu2.nc.gz','n2.nc.gz','n4.nc.gz','nu2.nc.gz','o1.nc.gz','p1.nc.gz',
             'q1.nc.gz','r2.nc.gz','s1.nc.gz','s2.nc.gz','s4.nc.gz','sa.nc.gz',
             'ssa.nc.gz','t2.nc.gz']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         c = ['2n2','eps2','j1','k1','k2','l2','lambda2','m2','m3','m4','m6','m8',
             'mf','mks2','mm','mn4','ms4','msf','msqm','mtm','mu2','n2','n4',
             'nu2','o1','p1','q1','r2','s1','s2','s4','sa','ssa','t2']
         model_format = 'FES'
         TYPE = 'z'
         SCALE = 1.0/100.0
+        GZIP = True
     elif (MODEL == 'FES2014_load'):
         model_directory = os.path.join(DIRECTORY,'fes2014','load_tide')
         model_files = ['2n2.nc.gz','eps2.nc.gz','j1.nc.gz','k1.nc.gz',
@@ -295,12 +332,14 @@ def compute_tide_corrections(x, y, delta_time, DIRECTORY=None, MODEL=None,
             'mu2.nc.gz','n2.nc.gz','n4.nc.gz','nu2.nc.gz','o1.nc.gz','p1.nc.gz',
             'q1.nc.gz','r2.nc.gz','s1.nc.gz','s2.nc.gz','s4.nc.gz','sa.nc.gz',
             'ssa.nc.gz','t2.nc.gz']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         c = ['2n2','eps2','j1','k1','k2','l2','lambda2','m2','m3','m4','m6',
             'm8','mf','mks2','mm','mn4','ms4','msf','msqm','mtm','mu2','n2',
             'n4','nu2','o1','p1','q1','r2','s1','s2','s4','sa','ssa','t2']
         model_format = 'FES'
         model_type = 'z'
         SCALE = 1.0/100.0
+        GZIP = True
     else:
         raise Exception("Unlisted tide model")
 
@@ -336,27 +375,30 @@ def compute_tide_corrections(x, y, delta_time, DIRECTORY=None, MODEL=None,
     #-- convert time to days relative to Jan 1, 1992 (48622mjd)
     t = pyTMD.time.convert_delta_time(delta_time - leap_seconds, epoch1=EPOCH,
         epoch2=(1992,1,1,0,0,0), scale=(1.0/86400.0))
+    #-- delta time (TT - UT1) file
+    delta_file = pyTMD.utilities.get_data_path(['data','merged_deltat.data'])
 
     #-- read tidal constants and interpolate to grid points
     if model_format in ('OTIS','ATLAS'):
         amp,ph,D,c = extract_tidal_constants(lon, lat, grid_file, model_file,
-            model_EPSG, TYPE=model_type, METHOD=METHOD, GRID=model_format)
+            model_EPSG, TYPE=model_type, METHOD=METHOD, EXTRAPOLATE=EXTRAPOLATE,
+            GRID=model_format)
         deltat = np.zeros_like(t)
     elif (model_format == 'netcdf'):
-        amp,ph,D,c = extract_netcdf_constants(lon, lat, model_directory,
-            grid_file, model_files, TYPE=model_type, METHOD=METHOD, SCALE=SCALE)
+        amp,ph,D,c = extract_netcdf_constants(lon, lat, grid_file, model_file,
+            TYPE=model_type, METHOD=METHOD, EXTRAPOLATE=EXTRAPOLATE,
+            SCALE=SCALE, GZIP=GZIP)
         deltat = np.zeros_like(t)
     elif (model_format == 'GOT'):
-        amp,ph = extract_GOT_constants(lon, lat, model_directory, model_files,
-            METHOD=METHOD, SCALE=SCALE)
+        amp,ph,c = extract_GOT_constants(lon, lat, model_file, METHOD=METHOD,
+            EXTRAPOLATE=EXTRAPOLATE, SCALE=SCALE, GZIP=GZIP)
         #-- interpolate delta times from calendar dates to tide time
-        delta_file = pyTMD.utilities.get_data_path(['data','merged_deltat.data'])
         deltat = calc_delta_time(delta_file, t)
     elif (model_format == 'FES'):
-        amp,ph = extract_FES_constants(lon, lat, model_directory, model_files,
-            TYPE=model_type, VERSION=MODEL, METHOD=METHOD, SCALE=SCALE)
+        amp,ph = extract_FES_constants(lon, lat, model_file, TYPE=model_type,
+            VERSION=MODEL, METHOD=METHOD, EXTRAPOLATE=EXTRAPOLATE,
+            SCALE=SCALE, GZIP=GZIP)
         #-- interpolate delta times from calendar dates to tide time
-        delta_file = pyTMD.utilities.get_data_path(['data','merged_deltat.data'])
         deltat = calc_delta_time(delta_file, t)
 
     #-- calculate complex phase in radians for Euler's
@@ -368,7 +410,7 @@ def compute_tide_corrections(x, y, delta_time, DIRECTORY=None, MODEL=None,
     if (TYPE.lower() == 'grid'):
         ny,nx = np.shape(x); nt = len(t)
         tide = np.ma.zeros((ny,nx,nt),fill_value=FILL_VALUE)
-        tide.mask = np.zeros((ny,nx,nt),dtype=np.bool)
+        tide.mask = np.zeros((ny,nx,nt),dtype=bool)
         for i in range(nt):
             TIDE = predict_tide(t[i], hc, c,
                 DELTAT=deltat[i], CORRECTIONS=model_format)
