@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 u"""
-read_netcdf_model.py (09/2020)
+read_netcdf_model.py (03/2021)
 Reads files for a tidal model and makes initial calculations to run tide program
 Includes functions to extract tidal harmonic constants from OTIS tide models for
     given locations
@@ -14,7 +14,6 @@ Reads netCDF4 ATLAS tidal solutions provided by Ohio State University and ESR
 INPUTS:
     ilon: longitude to interpolate
     ilat: latitude to interpolate
-    directory: data directory for tide data files
     grid_file: grid file for model (can be gzipped)
     model_files: list of model files for each constituent (can be gzipped)
 
@@ -29,6 +28,7 @@ OPTIONS:
         bilinear: quick bilinear interpolation
         spline: scipy bivariate spline interpolation
         linear, nearest: scipy regular grid interpolations
+    EXTRAPOLATE: extrapolate model using nearest-neighbors
     GZIP: input netCDF4 files are compressed
     SCALE: scaling factor for converting to output units
 
@@ -48,12 +48,21 @@ PYTHON DEPENDENCIES:
          https://unidata.github.io/netcdf4-python/netCDF4/index.html
 
 PROGRAM DEPENDENCIES:
-    bilinear_interp.py: bilinear interpolation of data to specified coordinates
+    bilinear_interp.py: bilinear interpolation of data to coordinates
+    nearest_extrap.py: nearest-neighbor extrapolation of data to coordinates
 
 UPDATE HISTORY:
+    Updated 03/2021: add extrapolation check where there are no invalid points
+        prevent ComplexWarning for fill values when calculating amplitudes
+        simplified inputs to be similar to binary OTIS read program
+    Updated 02/2021: set invalid values to nan in extrapolation
+        replaced numpy bool to prevent deprecation warning
+    Updated 12/2020: added valid data extrapolation with nearest_extrap
+        replace tostring with tobytes to fix DeprecationWarning
+    Updated 11/2020: create function to read bathymetry and spatial coordinates
     Updated 09/2020: set bounds error to false for regular grid interpolations
         adjust dimensions of input coordinates to be iterable
-        reduce number of interpolations by copying bathmetry mask to variables
+        reduce number of interpolations by copying bathymetry mask to variables
     Updated 08/2020: replaced griddata with scipy regular grid interpolators
     Updated 07/2020: added function docstrings. separate bilinear interpolation
         changed TYPE variable to keyword argument. update griddata interpolation
@@ -62,14 +71,16 @@ UPDATE HISTORY:
 """
 import os
 import gzip
+import uuid
 import netCDF4
 import numpy as np
 import scipy.interpolate
 from pyTMD.bilinear_interp import bilinear_interp
+from pyTMD.nearest_extrap import nearest_extrap
 
 #-- PURPOSE: extract tidal harmonic constants from tide models at coordinates
-def extract_netcdf_constants(ilon, ilat, directory, grid_file, model_files,
-    TYPE='z', METHOD='spline', GZIP=True, SCALE=1):
+def extract_netcdf_constants(ilon, ilat, grid_file, model_files, TYPE='z',
+    METHOD='spline', EXTRAPOLATE=False, GZIP=True, SCALE=1.0):
     """
     Reads files for a netCDF4 tidal model
     Makes initial calculations to run the tide program
@@ -79,22 +90,22 @@ def extract_netcdf_constants(ilon, ilat, directory, grid_file, model_files,
     ---------
     ilon: longitude to interpolate
     ilat: latitude to interpolate
-    directory: data directory for tide data files
     grid_file: grid file for model (can be gzipped)
     model_files: list of model files for each constituent (can be gzipped)
-    TYPE: tidal variable to run
+
+    Keyword arguments
+    -----------------
+    TYPE: tidal variable to read
         z: heights
         u: horizontal transport velocities
         U: horizontal depth-averaged transport
         v: vertical transport velocities
         V: vertical depth-averaged transport
-
-    Keyword arguments
-    -----------------
     METHOD: interpolation method
         bilinear: quick bilinear interpolation
         spline: scipy bivariate spline interpolation
         linear, nearest: scipy regular grid interpolations
+    EXTRAPOLATE: extrapolate model using nearest-neighbors
     GZIP: input netCDF4 files are compressed
     SCALE: scaling factor for converting to output units
 
@@ -106,43 +117,8 @@ def extract_netcdf_constants(ilon, ilat, directory, grid_file, model_files,
     constituents: list of model constituents
     """
 
-    #-- read the netcdf format tide grid file
-    #-- reading a combined global solution with localized solutions
-    if GZIP:
-        #-- open remote file with netCDF4
-        #-- read GZIP file
-        f = gzip.open(os.path.join(directory,grid_file),'rb')
-        fileID = netCDF4.Dataset(grid_file,'r',memory=f.read())
-    else:
-        fileID = netCDF4.Dataset(os.path.join(directory,grid_file),'r')
-    #-- variable dimensions
-    nx = fileID.dimensions['nx'].size
-    ny = fileID.dimensions['ny'].size
-    #-- allocate numpy masked array for bathymetry
-    bathymetry = np.ma.zeros((ny,nx))
-    #-- read bathmetry and coordinates for variable type
-    if (TYPE == 'z'):
-        #-- get bathymetry at nodes
-        bathymetry.data[:,:] = fileID.variables['hz'][:,:].T
-        #-- read latitude and longitude at z-nodes
-        lon = fileID.variables['lon_z'][:].copy()
-        lat = fileID.variables['lat_z'][:].copy()
-    elif TYPE in ('U','u'):
-        #-- get bathymetry at u-nodes
-        bathymetry.data[:,:] = fileID.variables['hu'][:,:].T
-        #-- read latitude and longitude at u-nodes
-        lon = fileID.variables['lon_u'][:].copy()
-        lat = fileID.variables['lat_u'][:].copy()
-    elif TYPE in ('V','v'):
-        #-- get bathymetry at v-nodes
-        bathymetry.data[:,:] = fileID.variables['hv'][:,:].T
-        #-- read latitude and longitude at v-nodes
-        lon = fileID.variables['lon_v'][:].copy()
-        lat = fileID.variables['lat_v'][:].copy()
-    #-- close the grid file
-    fileID.close()
-    f.close() if GZIP else None
-
+    #-- read the tide grid file for bathymetry and spatial coordinates
+    lon,lat,bathymetry = read_netcdf_grid(grid_file, TYPE, GZIP=GZIP)
     #-- grid step size of tide model
     dlon = lon[1] - lon[0]
     dlat = lat[1] - lat[0]
@@ -151,8 +127,6 @@ def extract_netcdf_constants(ilon, ilat, directory, grid_file, model_files,
     bathymetry = extend_matrix(bathymetry)
     #-- create masks
     bathymetry.mask = (bathymetry.data == 0)
-    #-- create meshes from latitude and longitude
-    gridlon,gridlat = np.meshgrid(lon,lat)
 
     #-- adjust dimensions of input coordinates to be iterable
     ilon = np.atleast_1d(ilon)
@@ -166,7 +140,7 @@ def extract_netcdf_constants(ilon, ilat, directory, grid_file, model_files,
 
     #-- interpolate bathymetry and mask to output points
     D = np.ma.zeros((npts))
-    D.mask = np.zeros((npts),dtype=np.bool)
+    D.mask = np.zeros((npts),dtype=bool)
     if (METHOD == 'bilinear'):
         #-- replace invalid values with nan
         bathymetry[bathymetry.mask] = np.nan
@@ -182,7 +156,7 @@ def extract_netcdf_constants(ilon, ilat, directory, grid_file, model_files,
         f2 = scipy.interpolate.RectBivariateSpline(lon,lat,
             bathymetry.mask.T,kx=1,ky=1)
         D.data[:] = f1.ev(ilon,ilat)
-        D.mask[:] = np.ceil(f2.ev(ilon,ilat).astype(np.bool))
+        D.mask[:] = np.ceil(f2.ev(ilon,ilat).astype(bool))
     else:
         #-- use scipy regular grid to interpolate values for a given method
         r1 = scipy.interpolate.RegularGridInterpolator((lat,lon),
@@ -190,7 +164,7 @@ def extract_netcdf_constants(ilon, ilat, directory, grid_file, model_files,
         r2 = scipy.interpolate.RegularGridInterpolator((lat,lon),
             bathymetry.mask, method=METHOD, bounds_error=False, fill_value=1)
         D.data[:] = r1.__call__(np.c_[ilat,ilon])
-        D.mask[:] = np.ceil(r2.__call__(np.c_[ilat,ilon])).astype(np.bool)
+        D.mask[:] = np.ceil(r2.__call__(np.c_[ilat,ilon])).astype(bool)
 
     #-- u and v are velocities in cm/s
     if TYPE in ('v','u'):
@@ -205,22 +179,24 @@ def extract_netcdf_constants(ilon, ilat, directory, grid_file, model_files,
     constituents = []
     #-- amplitude and phase
     ampl = np.ma.zeros((npts,nc))
-    ampl.mask = np.zeros((npts,nc),dtype=np.bool)
-    phase = np.ma.zeros((npts,nc))
-    phase.mask = np.zeros((npts,nc),dtype=np.bool)
+    ampl.mask = np.zeros((npts,nc),dtype=bool)
+    ph = np.ma.zeros((npts,nc))
+    ph.mask = np.zeros((npts,nc),dtype=bool)
     #-- read and interpolate each constituent
-    for i,fi in enumerate(model_files):
+    for i,model_file in enumerate(model_files):
         if (TYPE == 'z'):
             #-- read constituent from elevation file
-            z,con = read_elevation_file(os.path.join(directory,fi),GZIP)
+            z,con = read_elevation_file(model_file, GZIP=GZIP)
             #-- append constituent to list
             constituents.append(con)
             #-- replace original values with extend matrices
             z = extend_matrix(z)
             #-- interpolate amplitude and phase of the constituent
             z1 = np.ma.zeros((npts),dtype=z.dtype)
-            z1.mask = np.zeros((npts),dtype=np.bool)
+            z1.mask = np.zeros((npts),dtype=bool)
             if (METHOD == 'bilinear'):
+                #-- replace invalid values with nan
+                z[z.mask] = np.nan
                 z1.data[:] = bilinear_interp(lon,lat,z,ilon,ilat,dtype=z.dtype)
                 #-- mask invalid values
                 z1.mask[:] |= np.copy(D.mask)
@@ -244,19 +220,33 @@ def extract_netcdf_constants(ilon, ilat, directory, grid_file, model_files,
                 #-- mask invalid values
                 z1.mask[:] |= np.copy(D.mask)
                 z1.data[z1.mask] = z1.fill_value
+            #-- extrapolate data using nearest-neighbors
+            if EXTRAPOLATE and np.any(z1.mask):
+                #-- find invalid data points
+                inv, = np.nonzero(z1.mask)
+                #-- replace invalid values with nan
+                z[z.mask] = np.nan
+                #-- extrapolate points within 10km of valid model points
+                z1.data[inv] = nearest_extrap(lon,lat,z,ilon[inv],ilat[inv],
+                    dtype=z.dtype,cutoff=10.0)
+                #-- replace nan values with fill_value
+                z1.mask[inv] = np.isnan(z1.data[inv])
+                z1.data[z1.mask] = z1.fill_value
             #-- amplitude and phase of the constituent
-            ampl[:,i] = np.abs(z1)
-            phase[:,i] = np.arctan2(-np.imag(z1),np.real(z1))
+            ampl.data[:,i] = np.abs(z1.data)
+            ampl.mask[:,i] = np.copy(z1.mask)
+            ph.data[:,i] = np.arctan2(-np.imag(z1.data),np.real(z1.data))
+            ph.mask[:,i] = np.copy(z1.mask)
         elif TYPE in ('U','u','V','v'):
             #-- read constituent from transport file
-            tr,con = read_transport_file(os.path.join(directory,fi),TYPE,GZIP)
+            tr,con = read_transport_file(model_file, TYPE, GZIP=GZIP)
             #-- append constituent to list
             constituents.append(con)
             #-- replace original values with extend matrices
             tr = extend_matrix(tr)
             #-- interpolate amplitude and phase of the constituent
             tr1 = np.ma.zeros((npts),dtype=tr.dtype)
-            tr1.mask = np.zeros((npts),dtype=np.bool)
+            tr1.mask = np.zeros((npts),dtype=bool)
             if (METHOD == 'bilinear'):
                 tr1.data[:]=bilinear_interp(lon,lat,tr,ilon,ilat,dtype=tr.dtype)
                 #-- mask invalid values
@@ -281,16 +271,29 @@ def extract_netcdf_constants(ilon, ilat, directory, grid_file, model_files,
                 #-- mask invalid values
                 tr1.mask[:] |= np.copy(D.mask)
                 tr1.data[tr1.mask] = tr1.fill_value
+            #-- extrapolate data using nearest-neighbors
+            if EXTRAPOLATE and np.any(tr1.mask):
+                #-- find invalid data points
+                inv, = np.nonzero(tr1.mask)
+                #-- replace invalid values with nan
+                tr[tr.mask] = np.nan
+                #-- extrapolate points within 10km of valid model points
+                tr1.data[inv] = nearest_extrap(lon,lat,tr,ilon[inv],ilat[inv],
+                    dtype=tr.dtype,cutoff=10.0)
+                #-- replace nan values with fill_value
+                tr1.mask[inv] = np.isnan(tr1.data[inv])
+                tr1.data[tr1.mask] = tr1.fill_value
             #-- convert units
-            tr1 = tr1/unit_conv
             #-- amplitude and phase of the constituent
-            ampl[:,i] = np.abs(tr1)
-            phase[:,i] = np.arctan2(-np.imag(tr1),np.real(tr1))
+            ampl.data[:,i] = np.abs(tr1.data)/unit_conv
+            ampl.mask[:,i] = np.copy(tr1.mask)
+            ph.data[:,i] = np.arctan2(-np.imag(tr1.data),np.real(tr1.data))
+            ph.mask[:,i] = np.copy(tr1.mask)
 
     #-- convert amplitude from input units to meters
     amplitude = ampl*SCALE
     #-- convert phase to degrees
-    phase = phase*180.0/np.pi
+    phase = ph*180.0/np.pi
     phase[phase < 0] += 360.0
     #-- return the interpolated values
     return (amplitude,phase,D,constituents)
@@ -337,9 +340,73 @@ def extend_matrix(input_matrix):
     temp[:,-1] = input_matrix[:,0]
     return temp
 
+#-- PURPOSE: read grid file
+def read_netcdf_grid(input_file, TYPE, GZIP=False):
+    """
+    Read grid file to extract model coordinates and bathymetry
+
+    Arguments
+    ---------
+    input_file: input grid file
+    TYPE: tidal variable to run
+        z: heights
+        u: horizontal transport velocities
+        U: horizontal depth-averaged transport
+        v: vertical transport velocities
+        V: vertical depth-averaged transport
+
+    Keyword Arguments
+    -----------------
+    GZIP: input netCDF4 file is compressed
+
+    Returns
+    -------
+    lon: longitudinal coordinates of input grid
+    lat: latitudinal coordinates of input grid
+    bathymetry: model bathymetry
+    """
+    #-- read the netcdf format tide grid file
+    #-- reading a combined global solution with localized solutions
+    if GZIP:
+        #-- read GZIP file
+        f = gzip.open(os.path.expanduser(input_file),'rb')
+        fileID=netCDF4.Dataset(uuid.uuid4().hex,'r',memory=f.read())
+    else:
+        fileID=netCDF4.Dataset(os.path.expanduser(input_file),'r')
+    #-- variable dimensions
+    nx = fileID.dimensions['nx'].size
+    ny = fileID.dimensions['ny'].size
+    #-- allocate numpy masked array for bathymetry
+    bathymetry = np.ma.zeros((ny,nx))
+    #-- read bathymetry and coordinates for variable type
+    if (TYPE == 'z'):
+        #-- get bathymetry at nodes
+        bathymetry.data[:,:] = fileID.variables['hz'][:,:].T
+        #-- read latitude and longitude at z-nodes
+        lon = fileID.variables['lon_z'][:].copy()
+        lat = fileID.variables['lat_z'][:].copy()
+    elif TYPE in ('U','u'):
+        #-- get bathymetry at u-nodes
+        bathymetry.data[:,:] = fileID.variables['hu'][:,:].T
+        #-- read latitude and longitude at u-nodes
+        lon = fileID.variables['lon_u'][:].copy()
+        lat = fileID.variables['lat_u'][:].copy()
+    elif TYPE in ('V','v'):
+        #-- get bathymetry at v-nodes
+        bathymetry.data[:,:] = fileID.variables['hv'][:,:].T
+        #-- read latitude and longitude at v-nodes
+        lon = fileID.variables['lon_v'][:].copy()
+        lat = fileID.variables['lat_v'][:].copy()
+    #-- set bathymetry mask
+    bathymetry.mask = (bathymetry.data == 0.0)
+    #-- close the grid file
+    fileID.close()
+    f.close() if GZIP else None
+    return (lon,lat,bathymetry)
+
 #-- PURPOSE: read elevation file to extract real and imaginary components for
 #-- constituent
-def read_elevation_file(input_file,GZIP):
+def read_elevation_file(input_file, GZIP=False):
     """
     Read elevation file to extract real and imaginary components for constituent
 
@@ -359,18 +426,18 @@ def read_elevation_file(input_file,GZIP):
     #-- read the netcdf format tide elevation file
     #-- reading a combined global solution with localized solutions
     if GZIP:
-        f = gzip.open(input_file,'rb')
-        fileID = netCDF4.Dataset(input_file,'r',memory=f.read())
+        f = gzip.open(os.path.expanduser(input_file),'rb')
+        fileID = netCDF4.Dataset(uuid.uuid4().hex,'r',memory=f.read())
     else:
-        fileID = netCDF4.Dataset(input_file,'r')
+        fileID = netCDF4.Dataset(os.path.expanduser(input_file),'r')
     #-- constituent name
-    con = fileID.variables['con'][:].tostring().decode('utf-8')
+    con = fileID.variables['con'][:].tobytes().decode('utf-8')
     #-- variable dimensions
     nx = fileID.dimensions['nx'].size
     ny = fileID.dimensions['ny'].size
     #-- real and imaginary components of elevation
     h = np.ma.zeros((ny,nx),dtype=np.complex64)
-    h.mask = np.zeros((ny,nx),dtype=np.bool)
+    h.mask = np.zeros((ny,nx),dtype=bool)
     h.data.real[:,:] = fileID.variables['hRe'][:,:].T
     h.data.imag[:,:] = fileID.variables['hIm'][:,:].T
     #-- close the file
@@ -381,21 +448,21 @@ def read_elevation_file(input_file,GZIP):
 
 #-- PURPOSE: read transport file to extract real and imaginary components for
 #-- constituent
-def read_transport_file(input_file,TYPE,GZIP):
+def read_transport_file(input_file, TYPE, GZIP=False):
     """
     Read transport file to extract real and imaginary components for constituent
 
     Arguments
     ---------
     input_file: input transport file
-
-    Keyword arguments
-    -----------------
     TYPE: tidal variable to run
         u: horizontal transport velocities
         U: horizontal depth-averaged transport
         v: vertical transport velocities
         V: vertical depth-averaged transport
+
+    Keyword arguments
+    -----------------
     GZIP: input netCDF4 files are compressed
 
     Returns
@@ -406,18 +473,18 @@ def read_transport_file(input_file,TYPE,GZIP):
     #-- read the netcdf format tide grid file
     #-- reading a combined global solution with localized solutions
     if GZIP:
-        f = gzip.open(input_file,'rb')
-        fileID = netCDF4.Dataset(input_file,'r',memory=f.read())
+        f = gzip.open(os.path.expanduser(input_file),'rb')
+        fileID = netCDF4.Dataset(uuid.uuid4().hex,'r',memory=f.read())
     else:
-        fileID = netCDF4.Dataset(input_file,'r')
+        fileID = netCDF4.Dataset(os.path.expanduser(input_file),'r')
     #-- constituent name
-    con = fileID.variables['con'][:].tostring().decode('utf-8')
+    con = fileID.variables['con'][:].tobytes().decode('utf-8')
     #-- variable dimensions
     nx = fileID.dimensions['nx'].size
     ny = fileID.dimensions['ny'].size
     #-- real and imaginary components of transport
     tr = np.ma.zeros((ny,nx),dtype=np.complex64)
-    tr.mask = np.zeros((ny,nx),dtype=np.bool)
+    tr.mask = np.zeros((ny,nx),dtype=bool)
     if TYPE in ('U','u'):
         tr.data.real[:,:] = fileID.variables['uRe'][:,:].T
         tr.data.imag[:,:] = fileID.variables['uIm'][:,:].T
