@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 u"""
 compute_LPET_ICESat2_ATL06.py
-Written by Tyler Sutterley (11/2020)
+Written by Tyler Sutterley (04/2021)
 Calculates long-period equilibrium tidal elevations for correcting ICESat-2
     land ice elevation data
 Will calculate the long-period tides for all ATL06 segments and not just ocean
@@ -26,11 +26,14 @@ PROGRAM DEPENDENCIES:
     read_ICESat2_ATL06.py: reads ICESat-2 land ice along-track height data files
     time.py: utilities for calculating time operations
     utilities: download and management utilities for syncing files
-    convert_julian.py: returns the calendar date and time given a Julian date
     calc_delta_time.py: calculates difference between universal and dynamic time
     compute_equilibrium_tide.py: calculates long-period equilibrium ocean tides
 
 UPDATE HISTORY:
+    Updated 04/2021: can use a generically named ATL06 file as input
+    Updated 03/2021: replaced numpy bool/int to prevent deprecation warnings
+    Updated 12/2020: H5py deprecation warning change to use make_scale
+        merged time conversion routines into module
     Written 11/2020
 """
 from __future__ import print_function
@@ -44,24 +47,33 @@ import datetime
 import numpy as np
 import pyTMD.time
 from pyTMD.utilities import get_data_path
-from pyTMD.convert_julian import convert_julian
 from pyTMD.calc_delta_time import calc_delta_time
 from pyTMD.compute_equilibrium_tide import compute_equilibrium_tide
 from icesat2_toolkit.read_ICESat2_ATL06 import read_HDF5_ATL06
 
 #-- PURPOSE: read ICESat-2 land ice data (ATL06) from NSIDC
 #-- compute long-period equilibrium tides at points and times
-def compute_LPET_ICESat2(FILE, VERBOSE=False, MODE=0o775):
+def compute_LPET_ICESat2(INPUT_FILE, VERBOSE=False, MODE=0o775):
 
-    #-- read data from FILE
-    print('{0} -->'.format(os.path.basename(FILE))) if VERBOSE else None
-    IS2_atl06_mds,IS2_atl06_attrs,IS2_atl06_beams = read_HDF5_ATL06(FILE,
+    #-- read data from input file
+    print('{0} -->'.format(os.path.basename(INPUT_FILE))) if VERBOSE else None
+    IS2_atl06_mds,IS2_atl06_attrs,IS2_atl06_beams = read_HDF5_ATL06(INPUT_FILE,
         ATTRIBUTES=True)
-    DIRECTORY = os.path.dirname(FILE)
+    DIRECTORY = os.path.dirname(INPUT_FILE)
     #-- extract parameters from ICESat-2 ATLAS HDF5 file name
     rx = re.compile(r'(processed_)?(ATL\d{2})_(\d{4})(\d{2})(\d{2})(\d{2})'
-        '(\d{2})(\d{2})_(\d{4})(\d{2})(\d{2})_(\d{3})_(\d{2})(.*?).h5$')
-    SUB,PRD,YY,MM,DD,HH,MN,SS,TRK,CYCL,GRAN,RL,VERS,AUX = rx.findall(FILE).pop()
+        r'(\d{2})(\d{2})_(\d{4})(\d{2})(\d{2})_(\d{3})_(\d{2})(.*?).h5$')
+    try:
+        SUB,PRD,YY,MM,DD,HH,MN,SS,TRK,CYCL,GRAN,RL,VERS,AUX = rx.findall(INPUT_FILE).pop()
+    except:
+        #-- output long-period equilibrium tide HDF5 file (generic)
+        fileBasename,fileExtension = os.path.splitext(INPUT_FILE)
+        OUTPUT_FILE = '{0}_{1}{2}'.format(fileBasename,'LPET',fileExtension)
+    else:
+        #-- output long-period equilibrium tide HDF5 file for ASAS/NSIDC granules
+        args = (PRD,YY,MM,DD,HH,MN,SS,TRK,CYCL,GRAN,RL,VERS,AUX)
+        file_format = '{0}_LPET_{1}{2}{3}{4}{5}{6}_{7}{8}{9}_{10}_{11}{12}.h5'
+        OUTPUT_FILE = file_format.format(*args)
 
     #-- number of GPS seconds between the GPS epoch
     #-- and ATLAS Standard Data Product (SDP) epoch
@@ -70,6 +82,7 @@ def compute_LPET_ICESat2(FILE, VERBOSE=False, MODE=0o775):
     #-- copy variables for outputting to HDF5 file
     IS2_atl06_tide = {}
     IS2_atl06_fill = {}
+    IS2_atl06_dims = {}
     IS2_atl06_tide_attrs = {}
     #-- number of GPS seconds between the GPS epoch (1980-01-06T00:00:00Z UTC)
     #-- and ATLAS Standard Data Product (SDP) epoch (2018-01-01T00:00:00Z UTC)
@@ -89,6 +102,7 @@ def compute_LPET_ICESat2(FILE, VERBOSE=False, MODE=0o775):
         #-- output data dictionaries for beam
         IS2_atl06_tide[gtx] = dict(land_ice_segments={})
         IS2_atl06_fill[gtx] = dict(land_ice_segments={})
+        IS2_atl06_dims[gtx] = dict(land_ice_segments={})
         IS2_atl06_tide_attrs[gtx] = dict(land_ice_segments={})
 
         #-- number of segments
@@ -135,6 +149,7 @@ def compute_LPET_ICESat2(FILE, VERBOSE=False, MODE=0o775):
             mask=(val['delta_time']==fv))
         IS2_atl06_tide[gtx]['land_ice_segments']['delta_time'] = delta_time
         IS2_atl06_fill[gtx]['land_ice_segments']['delta_time'] = delta_time.fill_value
+        IS2_atl06_dims[gtx]['land_ice_segments']['delta_time'] = None
         IS2_atl06_tide_attrs[gtx]['land_ice_segments']['delta_time'] = {}
         IS2_atl06_tide_attrs[gtx]['land_ice_segments']['delta_time']['units'] = "seconds since 2018-01-01"
         IS2_atl06_tide_attrs[gtx]['land_ice_segments']['delta_time']['long_name'] = "Elapsed GPS seconds"
@@ -153,6 +168,7 @@ def compute_LPET_ICESat2(FILE, VERBOSE=False, MODE=0o775):
             mask=(val['latitude']==fv))
         IS2_atl06_tide[gtx]['land_ice_segments']['latitude'] = latitude
         IS2_atl06_fill[gtx]['land_ice_segments']['latitude'] = latitude.fill_value
+        IS2_atl06_dims[gtx]['land_ice_segments']['latitude'] = ['delta_time']
         IS2_atl06_tide_attrs[gtx]['land_ice_segments']['latitude'] = {}
         IS2_atl06_tide_attrs[gtx]['land_ice_segments']['latitude']['units'] = "degrees_north"
         IS2_atl06_tide_attrs[gtx]['land_ice_segments']['latitude']['contentType'] = "physicalMeasurement"
@@ -169,6 +185,7 @@ def compute_LPET_ICESat2(FILE, VERBOSE=False, MODE=0o775):
             mask=(val['longitude']==fv))
         IS2_atl06_tide[gtx]['land_ice_segments']['longitude'] = longitude
         IS2_atl06_fill[gtx]['land_ice_segments']['longitude'] = longitude.fill_value
+        IS2_atl06_dims[gtx]['land_ice_segments']['longitude'] = ['delta_time']
         IS2_atl06_tide_attrs[gtx]['land_ice_segments']['longitude'] = {}
         IS2_atl06_tide_attrs[gtx]['land_ice_segments']['longitude']['units'] = "degrees_east"
         IS2_atl06_tide_attrs[gtx]['land_ice_segments']['longitude']['contentType'] = "physicalMeasurement"
@@ -183,6 +200,7 @@ def compute_LPET_ICESat2(FILE, VERBOSE=False, MODE=0o775):
         #-- segment ID
         IS2_atl06_tide[gtx]['land_ice_segments']['segment_id'] = val['segment_id']
         IS2_atl06_fill[gtx]['land_ice_segments']['segment_id'] = None
+        IS2_atl06_dims[gtx]['land_ice_segments']['segment_id'] = ['delta_time']
         IS2_atl06_tide_attrs[gtx]['land_ice_segments']['segment_id'] = {}
         IS2_atl06_tide_attrs[gtx]['land_ice_segments']['segment_id']['units'] = "1"
         IS2_atl06_tide_attrs[gtx]['land_ice_segments']['segment_id']['contentType'] = "referenceInformation"
@@ -197,6 +215,7 @@ def compute_LPET_ICESat2(FILE, VERBOSE=False, MODE=0o775):
         #-- geophysical variables
         IS2_atl06_tide[gtx]['land_ice_segments']['geophysical'] = {}
         IS2_atl06_fill[gtx]['land_ice_segments']['geophysical'] = {}
+        IS2_atl06_dims[gtx]['land_ice_segments']['geophysical'] = {}
         IS2_atl06_tide_attrs[gtx]['land_ice_segments']['geophysical'] = {}
         IS2_atl06_tide_attrs[gtx]['land_ice_segments']['geophysical']['Description'] = ("The geophysical group "
             "contains parameters used to correct segment heights for geophysical effects, parameters "
@@ -206,6 +225,7 @@ def compute_LPET_ICESat2(FILE, VERBOSE=False, MODE=0o775):
         #-- computed long-period equilibrium tide
         IS2_atl06_tide[gtx]['land_ice_segments']['geophysical']['tide_equilibrium'] = tide_lpe
         IS2_atl06_fill[gtx]['land_ice_segments']['geophysical']['tide_equilibrium'] = tide_lpe.fill_value
+        IS2_atl06_dims[gtx]['land_ice_segments']['geophysical']['tide_equilibrium'] = ['delta_time']
         IS2_atl06_tide_attrs[gtx]['land_ice_segments']['geophysical']['tide_equilibrium'] = {}
         IS2_atl06_tide_attrs[gtx]['land_ice_segments']['geophysical']['tide_equilibrium']['units'] = "meters"
         IS2_atl06_tide_attrs[gtx]['land_ice_segments']['geophysical']['tide_equilibrium']['contentType'] = "referenceInformation"
@@ -218,20 +238,18 @@ def compute_LPET_ICESat2(FILE, VERBOSE=False, MODE=0o775):
         IS2_atl06_tide_attrs[gtx]['land_ice_segments']['geophysical']['tide_equilibrium']['coordinates'] = \
             "../segment_id ../delta_time ../latitude ../longitude"
 
-    #-- output equilibrium tide HDF5 file
-    args = (PRD,YY,MM,DD,HH,MN,SS,TRK,CYCL,GRAN,RL,VERS,AUX)
-    file_format = '{0}_LPET_{1}{2}{3}{4}{5}{6}_{7}{8}{9}_{10}_{11}{12}.h5'
     #-- print file information
-    print('\t{0}'.format(file_format.format(*args))) if VERBOSE else None
-    HDF5_ATL06_tide_write(IS2_atl06_tide, IS2_atl06_tide_attrs, CLOBBER=True,
-        INPUT=os.path.basename(FILE), FILL_VALUE=IS2_atl06_fill,
-        FILENAME=os.path.join(DIRECTORY,file_format.format(*args)))
+    print('\t{0}'.format(OUTPUT_FILE)) if VERBOSE else None
+    HDF5_ATL06_tide_write(IS2_atl06_tide, IS2_atl06_tide_attrs,
+        CLOBBER=True, INPUT=os.path.basename(INPUT_FILE),
+        FILL_VALUE=IS2_atl06_fill, DIMENSIONS=IS2_atl06_dims,
+        FILENAME=os.path.join(DIRECTORY,OUTPUT_FILE))
     #-- change the permissions mode
-    os.chmod(os.path.join(DIRECTORY,file_format.format(*args)), MODE)
+    os.chmod(os.path.join(DIRECTORY,OUTPUT_FILE), MODE)
 
 #-- PURPOSE: outputting the tide values for ICESat-2 data to HDF5
 def HDF5_ATL06_tide_write(IS2_atl06_tide, IS2_atl06_attrs, INPUT=None,
-    FILENAME='', FILL_VALUE=None, CLOBBER=False):
+    FILENAME='', FILL_VALUE=None, DIMENSIONS=None, CLOBBER=False):
     #-- setting HDF5 clobber attribute
     if CLOBBER:
         clobber = 'w'
@@ -272,34 +290,30 @@ def HDF5_ATL06_tide_write(IS2_atl06_tide, IS2_atl06_attrs, INPUT=None,
             att_val = IS2_atl06_attrs[gtx]['land_ice_segments'][att_name]
             fileID[gtx]['land_ice_segments'].attrs[att_name] = att_val
 
-        #-- delta_time
-        v = IS2_atl06_tide[gtx]['land_ice_segments']['delta_time']
-        attrs = IS2_atl06_attrs[gtx]['land_ice_segments']['delta_time']
-        #-- Defining the HDF5 dataset variables
-        val = '{0}/{1}/{2}'.format(gtx,'land_ice_segments','delta_time')
-        h5[gtx]['land_ice_segments']['delta_time'] = fileID.create_dataset(val,
-            np.shape(v), data=v, dtype=v.dtype, compression='gzip')
-        #-- add HDF5 variable attributes
-        for att_name,att_val in attrs.items():
-            h5[gtx]['land_ice_segments']['delta_time'].attrs[att_name] = att_val
-
-        #-- geolocation and segment_id variables
-        for k in ['latitude','longitude','segment_id']:
+        #-- delta_time, geolocation and segment_id variables
+        for k in ['delta_time','latitude','longitude','segment_id']:
             #-- values and attributes
             v = IS2_atl06_tide[gtx]['land_ice_segments'][k]
             attrs = IS2_atl06_attrs[gtx]['land_ice_segments'][k]
             fillvalue = FILL_VALUE[gtx]['land_ice_segments'][k]
             #-- Defining the HDF5 dataset variables
             val = '{0}/{1}/{2}'.format(gtx,'land_ice_segments',k)
-            h5[gtx]['land_ice_segments'][k] = fileID.create_dataset(val,
-                np.shape(v), data=v, dtype=v.dtype, fillvalue=fillvalue,
-                compression='gzip')
-            #-- attach dimensions
-            for dim in ['delta_time']:
-                h5[gtx]['land_ice_segments'][k].dims.create_scale(
-                    h5[gtx]['land_ice_segments'][dim], dim)
-                h5[gtx]['land_ice_segments'][k].dims[0].attach_scale(
-                    h5[gtx]['land_ice_segments'][dim])
+            if fillvalue:
+                h5[gtx]['land_ice_segments'][k] = fileID.create_dataset(val,
+                    np.shape(v), data=v, dtype=v.dtype, fillvalue=fillvalue,
+                    compression='gzip')
+            else:
+                h5[gtx]['land_ice_segments'][k] = fileID.create_dataset(val,
+                    np.shape(v), data=v, dtype=v.dtype, compression='gzip')
+            #-- create or attach dimensions for HDF5 variable
+            if DIMENSIONS[gtx]['land_ice_segments'][k]:
+                #-- attach dimensions
+                for i,dim in enumerate(DIMENSIONS[gtx]['land_ice_segments'][k]):
+                    h5[gtx]['land_ice_segments'][k].dims[i].attach_scale(
+                        h5[gtx]['land_ice_segments'][dim])
+            else:
+                #-- make dimension
+                h5[gtx]['land_ice_segments'][k].make_scale(k)
             #-- add HDF5 variable attributes
             for att_name,att_val in attrs.items():
                 h5[gtx]['land_ice_segments'][k].attrs[att_name] = att_val
@@ -326,10 +340,8 @@ def HDF5_ATL06_tide_write(IS2_atl06_tide, IS2_atl06_attrs, INPUT=None,
                     fileID.create_dataset(val, np.shape(v), data=v,
                     dtype=v.dtype, compression='gzip')
             #-- attach dimensions
-            for dim in ['delta_time']:
-                h5[gtx]['land_ice_segments'][key][k].dims.create_scale(
-                    h5[gtx]['land_ice_segments'][dim], dim)
-                h5[gtx]['land_ice_segments'][key][k].dims[0].attach_scale(
+            for i,dim in enumerate(DIMENSIONS[gtx]['land_ice_segments'][key][k]):
+                h5[gtx]['land_ice_segments'][key][k].dims[i].attach_scale(
                     h5[gtx]['land_ice_segments'][dim])
             #-- add HDF5 variable attributes
             for att_name,att_val in attrs.items():
@@ -337,7 +349,7 @@ def HDF5_ATL06_tide_write(IS2_atl06_tide, IS2_atl06_attrs, INPUT=None,
 
     #-- HDF5 file title
     fileID.attrs['featureType'] = 'trajectory'
-    fileID.attrs['title'] = 'ATLAS/ICESat-2 Land Ice Height'
+    fileID.attrs['title'] = 'ATLAS/ICESat-2 L3A Land Ice Height'
     fileID.attrs['summary'] = ('Estimates of the ice-sheet tidal parameters '
         'needed to interpret and assess the quality of land height estimates.')
     fileID.attrs['description'] = ('Land ice parameters for each beam.  All '
@@ -353,9 +365,9 @@ def HDF5_ATL06_tide_write(IS2_atl06_tide, IS2_atl06_attrs, INPUT=None,
     instrument = 'ATLAS > Advanced Topographic Laser Altimeter System'
     fileID.attrs['instrument'] = instrument
     fileID.attrs['source'] = 'Spacecraft'
-    fileID.attrs['references'] = 'http://nsidc.org/data/icesat2/data.html'
+    fileID.attrs['references'] = 'https://nsidc.org/data/icesat-2'
     fileID.attrs['processing_level'] = '4'
-    #-- add attributes for input ATL06 files
+    #-- add attributes for input ATL06 file
     fileID.attrs['input_files'] = os.path.basename(INPUT)
     #-- find geospatial and temporal ranges
     lnmn,lnmx,ltmn,ltmx,tmn,tmx = (np.inf,-np.inf,np.inf,-np.inf,np.inf,-np.inf)
@@ -388,14 +400,14 @@ def HDF5_ATL06_tide_write(IS2_atl06_tide, IS2_atl06_attrs, INPUT=None,
     #-- convert from seconds since 1980-01-06T00:00:00 to Julian days
     time_julian = 2400000.5 + pyTMD.time.convert_delta_time(gps_seconds - leaps,
         epoch1=(1980,1,6,0,0,0), epoch2=(1858,11,17,0,0,0), scale=1.0/86400.0)
-    #-- convert to calendar date with convert_julian.py
-    YY,MM,DD,HH,MN,SS = convert_julian(time_julian,FORMAT='tuple')
+    #-- convert to calendar date
+    YY,MM,DD,HH,MN,SS = pyTMD.time.convert_julian(time_julian,FORMAT='tuple')
     #-- add attributes with measurement date start, end and duration
-    tcs = datetime.datetime(np.int(YY[0]), np.int(MM[0]), np.int(DD[0]),
-        np.int(HH[0]), np.int(MN[0]), np.int(SS[0]), np.int(1e6*(SS[0] % 1)))
+    tcs = datetime.datetime(int(YY[0]), int(MM[0]), int(DD[0]),
+        int(HH[0]), int(MN[0]), int(SS[0]), int(1e6*(SS[0] % 1)))
     fileID.attrs['time_coverage_start'] = tcs.isoformat()
-    tce = datetime.datetime(np.int(YY[1]), np.int(MM[1]), np.int(DD[1]),
-        np.int(HH[1]), np.int(MN[1]), np.int(SS[1]), np.int(1e6*(SS[1] % 1)))
+    tce = datetime.datetime(int(YY[1]), int(MM[1]), int(DD[1]),
+        int(HH[1]), int(MN[1]), int(SS[1]), int(1e6*(SS[1] % 1)))
     fileID.attrs['time_coverage_end'] = tce.isoformat()
     fileID.attrs['time_coverage_duration'] = '{0:0.0f}'.format(tmx-tmn)
     #-- Closing the HDF5 file
