@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 u"""
 compute_LPT_displacements.py
-Written by Tyler Sutterley (11/2020)
+Written by Tyler Sutterley (03/2021)
 Calculates radial pole load tide displacements for an input file
     following IERS Convention (2010) guidelines
     http://maia.usno.navy.mil/conventions/2010officialinfo.php
@@ -26,7 +26,7 @@ COMMAND LINE OPTIONS:
     -t X, --type X: input data type
         drift: drift buoys or satellite/airborne altimetry (time per data point)
         grid: spatial grids or images (single time for all data points)
-    -E X, --epoch X: Reference epoch of input time (default Modified Julian Day)
+    -e X, --epoch X: Reference epoch of input time (default Modified Julian Day)
         days since 1858-11-17T00:00:00
     -d X, --deltatime X: Input delta time for files without date information
         can be set to 0 to use exact calendar date from epoch
@@ -56,12 +56,13 @@ PROGRAM DEPENDENCIES:
     time.py: utilities for calculating time operations
     spatial.py: utilities for reading and writing spatial data
     utilities: download and management utilities for syncing files
-    convert_julian.py: returns the calendar date and time given a Julian date
-    convert_calendar_decimal.py: converts from calendar dates into decimal years
     iers_mean_pole.py: provides the angular coordinates of IERS Mean Pole
     read_iers_EOP.py: read daily earth orientation parameters from IERS
 
 UPDATE HISTORY:
+    Updated 03/2021: use cartesian coordinate conversion routine in spatial
+    Updated 02/2021: replaced numpy bool to prevent deprecation warning
+    Updated 12/2020: merged time conversion routines into module
     Updated 11/2020: use internal mean pole and finals EOP files
         added options to read from and write to geotiff image files
     Updated 10/2020: using argparse to set command line parameters
@@ -79,15 +80,13 @@ import numpy as np
 import pyTMD.time
 import pyTMD.spatial
 import scipy.interpolate
-from pyTMD.convert_julian import convert_julian
-from pyTMD.convert_calendar_decimal import convert_calendar_decimal
 from pyTMD.iers_mean_pole import iers_mean_pole
 from pyTMD.read_iers_EOP import read_iers_EOP
 
 #-- PURPOSE: compute the pole load tide radial displacements following
 #-- IERS conventions (2010)
-def compute_LPT_displacements(input_file, output_file,
-    FORMAT='csv', VARIABLES=['time','lat','lon','data'], HEADER=0, TYPE='drift',
+def compute_LPT_displacements(input_file, output_file, FORMAT='csv',
+    VARIABLES=['time','lat','lon','data'], HEADER=0, TYPE='drift',
     TIME_UNITS='days since 1858-11-17T00:00:00', TIME=None, PROJECTION='4326',
     VERBOSE=False, MODE=0o775):
 
@@ -110,7 +109,7 @@ def compute_LPT_displacements(input_file, output_file,
     attrib['tide_pole']['description'] = ('Solid_Earth_pole_tide_radial_'
         'displacements_due_to_polar_motion')
     attrib['tide_pole']['reference'] = ('ftp://tai.bipm.org/iers/conv2010/'
-        'chapter7/opoleloadcoefcmcor.txt.gz')
+        'chapter7/tn36_c7.pdf')
     attrib['tide_pole']['units'] = 'meters'
     attrib['tide_pole']['_FillValue'] = fill_value
     #-- time
@@ -152,9 +151,9 @@ def compute_LPT_displacements(input_file, output_file,
     if (TYPE == 'grid'):
         ny,nx = (len(dinput['y']),len(dinput['x']))
         gridx,gridy = np.meshgrid(dinput['x'],dinput['y'])
-        longitude,lat = transformer.transform(gridx.flatten(),gridy.flatten())
+        lon,lat = transformer.transform(gridx.flatten(),gridy.flatten())
     elif (TYPE == 'drift'):
-        longitude,lat = transformer.transform(dinput['x'].flatten(),
+        lon,lat = transformer.transform(dinput['x'].flatten(),
             dinput['y'].flatten())
 
     #-- extract time units from netCDF4 and HDF5 attributes or from TIME_UNITS
@@ -168,9 +167,10 @@ def compute_LPT_displacements(input_file, output_file,
     MJD = pyTMD.time.convert_delta_time(to_secs*dinput['time'].flatten(),
         epoch1=epoch1, epoch2=(1858,11,17,0,0,0), scale=1.0/86400.0)
     #-- add offset to convert to Julian days and then convert to calendar dates
-    Y,M,D,h,m,s = convert_julian(2400000.5 + MJD, FORMAT='tuple')
+    Y,M,D,h,m,s = pyTMD.time.convert_julian(2400000.5 + MJD, FORMAT='tuple')
     #-- calculate time in year-decimal format
-    time_decimal = convert_calendar_decimal(Y,M,DAY=D,HOUR=h,MINUTE=m,SECOND=s)
+    time_decimal = pyTMD.time.convert_calendar_decimal(Y,M,day=D,
+        hour=h,minute=m,second=s)
     #-- number of time points
     nt = len(time_decimal)
 
@@ -199,20 +199,14 @@ def compute_LPT_displacements(input_file, output_file,
     #-- flatten heights
     h = dinput['data'].flatten() if ('data' in dinput.keys()) else 0.0
     #-- convert from geodetic latitude to geocentric latitude
-    #-- geodetic latitude in radians
-    latitude_geodetic_rad = lat*dtr
-    #-- prime vertical radius of curvature
-    N = a_axis/np.sqrt(1.0 - ecc1**2.0*np.sin(latitude_geodetic_rad)**2.0)
     #-- calculate X, Y and Z from geodetic latitude and longitude
-    X = (N+h)*np.cos(latitude_geodetic_rad)*np.cos(longitude*dtr)
-    Y = (N+h)*np.cos(latitude_geodetic_rad)*np.sin(longitude*dtr)
-    Z = (N * (1.0 - ecc1**2.0) + h) * np.sin(latitude_geodetic_rad)
+    X,Y,Z = pyTMD.spatial.to_cartesian(lon,lat,h=h,a_axis=a_axis,flat=flat)
     rr = np.sqrt(X**2.0 + Y**2.0 + Z**2.0)
     #-- calculate geocentric latitude and convert to degrees
     latitude_geocentric = np.arctan(Z / np.sqrt(X**2.0 + Y**2.0))/dtr
     #-- geocentric colatitude and longitude in radians
     theta = dtr*(90.0 - latitude_geocentric)
-    phi = longitude*dtr
+    phi = lon*dtr
 
     #-- compute normal gravity at spatial location and elevation of points.
     #-- normal gravity at the equator. p. 79, Eqn.(2-186)
@@ -252,7 +246,7 @@ def compute_LPT_displacements(input_file, output_file,
     #-- calculate radial displacement at time
     if (TYPE == 'grid'):
         Srad = np.ma.zeros((ny,nx,nt),fill_value=fill_value)
-        Srad.mask = np.zeros((ny,nx,nt),dtype=np.bool)
+        Srad.mask = np.zeros((ny,nx,nt),dtype=bool)
         for i in range(nt):
             SRAD=dfactor*np.sin(2.0*theta)*(mx[i]*np.cos(phi)+my[i]*np.sin(phi))
             #-- reform grid
@@ -266,7 +260,7 @@ def compute_LPT_displacements(input_file, output_file,
     Srad.data[Srad.mask] = Srad.fill_value
 
     #-- output to file
-    output = dict(time=MJD,lon=longitude,lat=lat,tide_pole=Srad)
+    output = dict(time=MJD,lon=lon,lat=lat,tide_pole=Srad)
     if (FORMAT == 'csv'):
         pyTMD.spatial.to_ascii(output, attrib, output_file, delimiter=',',
             columns=['time','lat','lon','tide_pole'], verbose=VERBOSE)
@@ -317,7 +311,7 @@ def main():
         help='Input data type')
     #-- time epoch (default Modified Julian Days)
     #-- in form "time-units since yyyy-mm-dd hh:mm:ss"
-    parser.add_argument('--epoch','-E',
+    parser.add_argument('--epoch','-e',
         type=str, default='days since 1858-11-17T00:00:00',
         help='Reference epoch of input time')
     #-- input delta time for files without date information
