@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 u"""
 compute_OPT_displacements.py
-Written by Tyler Sutterley (10/2020)
+Written by Tyler Sutterley (04/2021)
 Calculates radial ocean pole load tide displacements for an input file
     following IERS Convention (2010) guidelines
     http://maia.usno.navy.mil/conventions/2010officialinfo.php
@@ -26,7 +26,7 @@ COMMAND LINE OPTIONS:
     -t X, --type X: input data type
         drift: drift buoys or satellite/airborne altimetry (time per data point)
         grid: spatial grids or images (single time for all data points)
-    -E X, --epoch X: Reference epoch of input time (default Modified Julian Day)
+    -e X, --epoch X: Reference epoch of input time (default Modified Julian Day)
         days since 1858-11-17T00:00:00
     -d X, --deltatime X: Input delta time for files without date information
         can be set to 0 to use exact calendar date from epoch
@@ -60,13 +60,24 @@ PROGRAM DEPENDENCIES:
     time.py: utilities for calculating time operations
     spatial.py: utilities for reading and writing spatial data
     utilities: download and management utilities for syncing files
-    convert_julian.py: returns the calendar date and time given a Julian date
-    convert_calendar_decimal.py: converts from calendar dates into decimal years
     iers_mean_pole.py: provides the angular coordinates of IERS Mean Pole
     read_iers_EOP.py: read daily earth orientation parameters from IERS
     read_ocean_pole_tide.py: read ocean pole load tide map from IERS
 
+REFERENCES:
+    S Desai, "Observing the pole tide with satellite altimetry", Journal of
+        Geophysical Research: Oceans, 107(C11), 2002. doi: 10.1029/2001JC001224
+    S Desai, J Wahr and B Beckley "Revisiting the pole tide for and from
+        satellite altimetry", Journal of Geodesy, 89(12), p1233-1243, 2015.
+        doi: 10.1007/s00190-015-0848-7
+
 UPDATE HISTORY:
+    Updated 04/2021: fix arguments to program (using internal EOP files)
+        Add missing delta time argument for files without time variables
+        Thanks to Karen Alley for pointing these out!
+    Updated 03/2021: use cartesian coordinate conversion routine in spatial
+    Updated 02/2021: replaced numpy bool to prevent deprecation warning
+    Updated 12/2020: merged time conversion routines into module
     Updated 11/2020: use internal mean pole and finals EOP files
         added options to read from and write to geotiff image files
     Updated 10/2020: using argparse to set command line parameters
@@ -86,16 +97,14 @@ import pyTMD.time
 import pyTMD.spatial
 import scipy.interpolate
 from pyTMD.utilities import get_data_path
-from pyTMD.convert_julian import convert_julian
-from pyTMD.convert_calendar_decimal import convert_calendar_decimal
 from pyTMD.iers_mean_pole import iers_mean_pole
 from pyTMD.read_iers_EOP import read_iers_EOP
 from pyTMD.read_ocean_pole_tide import read_ocean_pole_tide
 
 #-- PURPOSE: compute the ocean pole load tide radial displacements following
 #-- IERS conventions (2010) and using data from Desai (2002)
-def compute_OPT_displacements(tide_dir, input_file, output_file,
-    FORMAT='csv', VARIABLES=['time','lat','lon','data'], HEADER=0, TYPE='drift',
+def compute_OPT_displacements(input_file, output_file, FORMAT='csv',
+    VARIABLES=['time','lat','lon','data'], HEADER=0, TYPE='drift',
     TIME_UNITS='days since 1858-11-17T00:00:00', TIME=None, PROJECTION='4326',
     METHOD='spline', VERBOSE=False, MODE=0o775):
 
@@ -160,9 +169,9 @@ def compute_OPT_displacements(tide_dir, input_file, output_file,
     if (TYPE == 'grid'):
         ny,nx = (len(dinput['y']),len(dinput['x']))
         gridx,gridy = np.meshgrid(dinput['x'],dinput['y'])
-        longitude,lat = transformer.transform(gridx.flatten(),gridy.flatten())
+        lon,lat = transformer.transform(gridx.flatten(),gridy.flatten())
     elif (TYPE == 'drift'):
-        longitude,lat = transformer.transform(dinput['x'].flatten(),
+        lon,lat = transformer.transform(dinput['x'].flatten(),
             dinput['y'].flatten())
 
     #-- extract time units from netCDF4 and HDF5 attributes or from TIME_UNITS
@@ -176,9 +185,10 @@ def compute_OPT_displacements(tide_dir, input_file, output_file,
     MJD = pyTMD.time.convert_delta_time(to_secs*dinput['time'].flatten(),
         epoch1=epoch1, epoch2=(1858,11,17,0,0,0), scale=1.0/86400.0)
     #-- add offset to convert to Julian days and then convert to calendar dates
-    Y,M,D,h,m,s = convert_julian(2400000.5 + MJD, FORMAT='tuple')
+    Y,M,D,h,m,s = pyTMD.time.convert_julian(2400000.5 + MJD, FORMAT='tuple')
     #-- calculate time in year-decimal format
-    time_decimal = convert_calendar_decimal(Y,M,DAY=D,HOUR=h,MINUTE=m,SECOND=s)
+    time_decimal = pyTMD.time.convert_calendar_decimal(Y,M,day=D,
+        hour=h,minute=m,second=s)
     #-- number of time points
     nt = len(time_decimal)
 
@@ -202,14 +212,8 @@ def compute_OPT_displacements(tide_dir, input_file, output_file,
     #-- flatten heights
     h = dinput['data'].flatten() if ('data' in dinput.keys()) else 0.0
     #-- convert from geodetic latitude to geocentric latitude
-    #-- geodetic latitude in radians
-    latitude_geodetic_rad = lat*dtr
-    #-- prime vertical radius of curvature
-    N = a_axis/np.sqrt(1.0 - ecc1**2.0*np.sin(latitude_geodetic_rad)**2.0)
     #-- calculate X, Y and Z from geodetic latitude and longitude
-    X = (N+h)*np.cos(latitude_geodetic_rad)*np.cos(longitude*dtr)
-    Y = (N+h)*np.cos(latitude_geodetic_rad)*np.sin(longitude*dtr)
-    Z = (N * (1.0 - ecc1**2.0) + h) * np.sin(latitude_geodetic_rad)
+    X,Y,Z = pyTMD.spatial.to_cartesian(lon,lat,h=h,a_axis=a_axis,flat=flat)
     #-- calculate geocentric latitude and convert to degrees
     latitude_geocentric = np.arctan(Z / np.sqrt(X**2.0 + Y**2.0))/dtr
 
@@ -219,8 +223,8 @@ def compute_OPT_displacements(tide_dir, input_file, output_file,
     K1 = 4.0*np.pi*G*rho_w*Hp*a_axis**3/(3.0*GM)
 
     #-- pole tide files (mean and daily)
-    mean_pole_file = pyTMD.utilities.get_data_path(['data','mean-pole.tab'])
-    pole_tide_file = pyTMD.utilities.get_data_path(['data','finals.all'])
+    mean_pole_file = get_data_path(['data','mean-pole.tab'])
+    pole_tide_file = get_data_path(['data','finals.all'])
     #-- calculate angular coordinates of mean pole at time
     mpx,mpy,fl = iers_mean_pole(mean_pole_file,time_decimal,'2015')
     #-- read IERS daily polar motion values
@@ -245,18 +249,18 @@ def compute_OPT_displacements(tide_dir, input_file, output_file,
         f2 = scipy.interpolate.RectBivariateSpline(ilon, ilat[::-1],
             iur[:,::-1].imag, kx=1, ky=1)
         UR = np.zeros((len(latitude_geocentric)),dtype=np.complex128)
-        UR.real = f1.ev(longitude,latitude_geocentric)
-        UR.imag = f2.ev(longitude,latitude_geocentric)
+        UR.real = f1.ev(lon,latitude_geocentric)
+        UR.imag = f2.ev(lon,latitude_geocentric)
     else:
         #-- use scipy regular grid to interpolate values for a given method
         r1 = scipy.interpolate.RegularGridInterpolator((ilon,ilat[::-1]),
             iur[:,::-1], method=METHOD)
-        UR = r1.__call__(np.c_[longitude,latitude_geocentric])
+        UR = r1.__call__(np.c_[lon,latitude_geocentric])
 
     #-- calculate radial displacement at time
     if (TYPE == 'grid'):
         Urad = np.ma.zeros((ny,nx,nt),fill_value=fill_value)
-        Urad.mask = np.zeros((ny,nx,nt),dtype=np.bool)
+        Urad.mask = np.zeros((ny,nx,nt),dtype=bool)
         for i in range(nt):
             URAD = K*atr*np.real((mx[i]*gamma.real + my[i]*gamma.imag)*UR.real +
                 (my[i]*gamma.real - mx[i]*gamma.imag)*UR.imag)
@@ -272,7 +276,7 @@ def compute_OPT_displacements(tide_dir, input_file, output_file,
     Urad.data[Urad.mask] = Urad.fill_value
 
     #-- output to file
-    output = dict(time=MJD,lon=longitude,lat=lat,tide_oc_pole=Urad)
+    output = dict(time=MJD,lon=lon,lat=lat,tide_oc_pole=Urad)
     if (FORMAT == 'csv'):
         pyTMD.spatial.to_ascii(output, attrib, output_file, delimiter=',',
             columns=['time','lat','lon','tide_oc_pole'], verbose=VERBOSE)
@@ -323,9 +327,13 @@ def main():
         help='Input data type')
     #-- time epoch (default Modified Julian Days)
     #-- in form "time-units since yyyy-mm-dd hh:mm:ss"
-    parser.add_argument('--epoch','-E',
+    parser.add_argument('--epoch','-e',
         type=str, default='days since 1858-11-17T00:00:00',
         help='Reference epoch of input time')
+    #-- input delta time for files without date information
+    parser.add_argument('--deltatime','-d',
+        type=float, nargs='+',
+        help='Input delta time for files without date variables')
     #-- spatial projection (EPSG code or PROJ4 string)
     parser.add_argument('--projection','-P',
         type=str, default='4326',
