@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 u"""
 compute_OPT_icebridge_data.py
-Written by Tyler Sutterley (10/2020)
+Written by Tyler Sutterley (03/2021)
 Calculates radial ocean pole tide displacements for correcting Operation
     IceBridge elevation data following IERS Convention (2010) guidelines
     http://maia.usno.navy.mil/conventions/2010officialinfo.php
@@ -29,15 +29,17 @@ PYTHON DEPENDENCIES:
 
 PROGRAM DEPENDENCIES:
     time.py: utilities for calculating time operations
+    spatial.py: utilities for reading, writing and operating on spatial data
     utilities: download and management utilities for syncing files
-    convert_julian.py: returns the calendar date and time given a Julian date
-    convert_calendar_decimal.py: converts from calendar dates into decimal years
     iers_mean_pole.py: provides the angular coordinates of IERS Mean Pole
     read_iers_EOP.py: read daily earth orientation parameters from IERS
     read_ocean_pole_tide.py: read ocean pole load tide map from IERS
     read_ATM1b_QFIT_binary.py: read ATM1b QFIT binary files (NSIDC version 1)
 
 UPDATE HISTORY:
+    Updated 03/2021: use cartesian coordinate conversion routine in spatial
+        replaced numpy bool/int to prevent deprecation warnings
+    Updated 12/2020: merged time conversion routines into module
     Updated 11/2020: use internal mean pole and finals EOP files
     Updated 10/2020: using argparse to set command line parameters
     Updated 09/2020: output modified julian days as time variable
@@ -61,9 +63,8 @@ import argparse
 import numpy as np
 import scipy.interpolate
 import pyTMD.time
+import pyTMD.spatial
 from pyTMD.utilities import get_data_path
-from pyTMD.convert_julian import convert_julian
-from pyTMD.convert_calendar_decimal import convert_calendar_decimal
 from pyTMD.iers_mean_pole import iers_mean_pole
 from pyTMD.read_iers_EOP import read_iers_EOP
 from pyTMD.read_ocean_pole_tide import read_ocean_pole_tide
@@ -118,22 +119,22 @@ def read_ATM_qfit_file(input_file, input_subsetter):
         #-- number of lines of data within file
         file_lines = file_length(input_file,input_subsetter)
         #-- create output variables with length equal to the number of lines
-        ATM_L1b_input['lat'] = np.zeros_like(file_contents,dtype=np.float)
-        ATM_L1b_input['lon'] = np.zeros_like(file_contents,dtype=np.float)
-        ATM_L1b_input['data'] = np.zeros_like(file_contents,dtype=np.float)
-        hour = np.zeros_like(file_contents,dtype=np.float)
-        minute = np.zeros_like(file_contents,dtype=np.float)
-        second = np.zeros_like(file_contents,dtype=np.float)
+        ATM_L1b_input['lat'] = np.zeros_like(file_contents,dtype=np.float64)
+        ATM_L1b_input['lon'] = np.zeros_like(file_contents,dtype=np.float64)
+        ATM_L1b_input['data'] = np.zeros_like(file_contents,dtype=np.float64)
+        hour = np.zeros_like(file_contents,dtype=np.float64)
+        minute = np.zeros_like(file_contents,dtype=np.float64)
+        second = np.zeros_like(file_contents,dtype=np.float64)
         #-- for each line within the file
         for i,line in enumerate(file_contents):
             #-- find numerical instances within the line
             line_contents = rx.findall(line)
-            ATM_L1b_input['lat'][i] = np.float(line_contents[1])
-            ATM_L1b_input['lon'][i] = np.float(line_contents[2])
-            ATM_L1b_input['data'][i] = np.float(line_contents[3])
-            hour[i] = np.float(line_contents[-1][:2])
-            minute[i] = np.float(line_contents[-1][2:4])
-            second[i] = np.float(line_contents[-1][4:])
+            ATM_L1b_input['lat'][i] = np.float64(line_contents[1])
+            ATM_L1b_input['lon'][i] = np.float64(line_contents[2])
+            ATM_L1b_input['data'][i] = np.float64(line_contents[3])
+            hour[i] = np.float64(line_contents[-1][:2])
+            minute[i] = np.float64(line_contents[-1][2:4])
+            second[i] = np.float64(line_contents[-1][4:])
     #-- Version 1 of ATM QFIT files (binary)
     elif (SFX == 'qi'):
         #-- read input QFIT data file and subset if specified
@@ -145,16 +146,16 @@ def read_ATM_qfit_file(input_file, input_subsetter):
         ATM_L1b_input['data'] = fid['elevation'][:]
         time_hhmmss = fid['time_hhmmss'][:]
         #-- extract hour, minute and second from time_hhmmss
-        hour = np.zeros_like(time_hhmmss,dtype=np.float)
-        minute = np.zeros_like(time_hhmmss,dtype=np.float)
-        second = np.zeros_like(time_hhmmss,dtype=np.float)
+        hour = np.zeros_like(time_hhmmss,dtype=np.float64)
+        minute = np.zeros_like(time_hhmmss,dtype=np.float64)
+        second = np.zeros_like(time_hhmmss,dtype=np.float64)
         #-- for each line within the file
         for i,packed_time in enumerate(time_hhmmss):
             #-- convert to zero-padded string with 3 decimal points
             line_contents = '{0:010.3f}'.format(packed_time)
-            hour[i] = np.float(line_contents[:2])
-            minute[i] = np.float(line_contents[2:4])
-            second[i] = np.float(line_contents[4:])
+            hour[i] = np.float64(line_contents[:2])
+            minute[i] = np.float64(line_contents[2:4])
+            second[i] = np.float64(line_contents[4:])
     #-- Version 2 of ATM QFIT files (HDF5)
     elif (SFX == 'h5'):
         #-- Open the HDF5 file for reading
@@ -167,16 +168,16 @@ def read_ATM_qfit_file(input_file, input_subsetter):
         ATM_L1b_input['data'] = fileID['elevation'][:]
         time_hhmmss = fileID['instrument_parameters']['time_hhmmss'][:]
         #-- extract hour, minute and second from time_hhmmss
-        hour = np.zeros_like(time_hhmmss,dtype=np.float)
-        minute = np.zeros_like(time_hhmmss,dtype=np.float)
-        second = np.zeros_like(time_hhmmss,dtype=np.float)
+        hour = np.zeros_like(time_hhmmss,dtype=np.float64)
+        minute = np.zeros_like(time_hhmmss,dtype=np.float64)
+        second = np.zeros_like(time_hhmmss,dtype=np.float64)
         #-- for each line within the file
         for i,packed_time in enumerate(time_hhmmss):
             #-- convert to zero-padded string with 3 decimal points
             line_contents = '{0:010.3f}'.format(packed_time)
-            hour[i] = np.float(line_contents[:2])
-            minute[i] = np.float(line_contents[2:4])
-            second[i] = np.float(line_contents[4:])
+            hour[i] = np.float64(line_contents[:2])
+            minute[i] = np.float64(line_contents[2:4])
+            second[i] = np.float64(line_contents[4:])
         #-- close the input HDF5 file
         fileID.close()
     #-- calculate the number of leap seconds between GPS time (seconds
@@ -231,14 +232,14 @@ def read_ATM_icessn_file(input_file, input_subsetter):
     ATM_L2_input = {}
     #-- create output variables with length equal to the number of file lines
     for key in file_dtype.keys():
-        ATM_L2_input[key] = np.zeros_like(file_contents, dtype=np.float)
+        ATM_L2_input[key] = np.zeros_like(file_contents, dtype=np.float64)
     #-- for each line within the file
     for line_number,line_entries in enumerate(file_contents):
         #-- find numerical instances within the line
         line_contents = rx.findall(line_entries)
         #-- for each variable of interest: save to dinput as float
         for key,val in file_dtype.items():
-            ATM_L2_input[key][line_number] = np.float(line_contents[val])
+            ATM_L2_input[key][line_number] = np.float64(line_contents[val])
     #-- convert shot time (seconds of day) to J2000
     hour = np.floor(ATM_L2_input['seconds']/3600.0)
     minute = np.floor((ATM_L2_input['seconds'] % 3600)/60.0)
@@ -283,7 +284,7 @@ def read_LVIS_HDF5_file(input_file, input_subsetter):
     regex_pattern = r'{0}_(.*?)(\d+)_(\d+)_(R\d+)_(\d+).H5'.format(mission_flag)
     #-- extract mission, region and other parameters from filename
     MISSION,REGION,YY,MMDD,RLD,SS = re.findall(regex_pattern,input_file).pop()
-    LDS_VERSION = '2.0.2' if (np.int(RLD[1:3]) >= 18) else '1.04'
+    LDS_VERSION = '2.0.2' if (int(RLD[1:3]) >= 18) else '1.04'
     #-- input and output python dictionaries with variables
     file_input = {}
     LVIS_L2_input = {}
@@ -328,10 +329,10 @@ def read_LVIS_HDF5_file(input_file, input_subsetter):
     #-- close the input HDF5 file
     fileID.close()
     #-- output combined variables
-    LVIS_L2_input['data'] = np.zeros_like(file_input['elev'],dtype=np.float)
-    LVIS_L2_input['lon'] = np.zeros_like(file_input['elev'],dtype=np.float)
-    LVIS_L2_input['lat'] = np.zeros_like(file_input['elev'],dtype=np.float)
-    LVIS_L2_input['error'] = np.zeros_like(file_input['elev'],dtype=np.float)
+    LVIS_L2_input['data'] = np.zeros_like(file_input['elev'],dtype=np.float64)
+    LVIS_L2_input['lon'] = np.zeros_like(file_input['elev'],dtype=np.float64)
+    LVIS_L2_input['lat'] = np.zeros_like(file_input['elev'],dtype=np.float64)
+    LVIS_L2_input['error'] = np.zeros_like(file_input['elev'],dtype=np.float64)
     #-- find where elev high is equal to elev low
     #-- see note about using LVIS centroid elevation product
     #-- http://lvis.gsfc.nasa.gov/OIBDataStructure.html
@@ -433,10 +434,10 @@ def compute_OPT_icebridge_data(arg,METHOD=None,VERBOSE=False,MODE=0o775):
         #-- early date strings omitted century and millenia (e.g. 93 for 1993)
         if (len(YYMMDD1) == 6):
             ypre,MM1,DD1 = YYMMDD1[:2],YYMMDD1[2:4],YYMMDD1[4:]
-            if (np.float(ypre) >= 90):
-                YY1 = '{0:4.0f}'.format(np.float(ypre) + 1900.0)
+            if (np.float64(ypre) >= 90):
+                YY1 = '{0:4.0f}'.format(np.float64(ypre) + 1900.0)
             else:
-                YY1 = '{0:4.0f}'.format(np.float(ypre) + 2000.0)
+                YY1 = '{0:4.0f}'.format(np.float64(ypre) + 2000.0)
         elif (len(YYMMDD1) == 8):
             YY1,MM1,DD1 = YYMMDD1[:4],YYMMDD1[4:6],YYMMDD1[6:]
     elif OIB in ('LVIS','LVGH'):
@@ -462,9 +463,10 @@ def compute_OPT_icebridge_data(arg,METHOD=None,VERBOSE=False,MODE=0o775):
     #-- J2000: seconds since 2000-01-01 12:00:00 UTC
     t = dinput['time'][:]/86400.0 + 51544.5
     #-- convert from MJD to calendar dates
-    YY,MM,DD,HH,MN,SS = convert_julian(t + 2400000.5,FORMAT='tuple')
+    YY,MM,DD,HH,MN,SS = pyTMD.time.convert_julian(t + 2400000.5,FORMAT='tuple')
     #-- convert calendar dates into year decimal
-    tdec = convert_calendar_decimal(YY,MM,DAY=DD,HOUR=HH,MINUTE=MN,SECOND=SS)
+    tdec = pyTMD.time.convert_calendar_decimal(YY,MM,day=DD,
+        hour=HH,minute=MN,second=SS)
     #-- elevation
     h1 = dinput['data'][:]
 
@@ -487,15 +489,8 @@ def compute_OPT_icebridge_data(arg,METHOD=None,VERBOSE=False,MODE=0o775):
     gamma = 0.6870 + 0.0036j
 
     #-- convert from geodetic latitude to geocentric latitude
-    #-- geodetic latitude in radians
-    latitude_geodetic_rad = lat*dtr
-    #-- prime vertical radius of curvature
-    N = a_axis/np.sqrt(1.0 - ecc1**2.0*np.sin(latitude_geodetic_rad)**2.0)
     #-- calculate X, Y and Z from geodetic latitude and longitude
-    X = (N + h1) * np.cos(latitude_geodetic_rad) * np.cos(lon*dtr)
-    Y = (N + h1) * np.cos(latitude_geodetic_rad) * np.sin(lon*dtr)
-    Z = (N * (1.0 - ecc1**2.0) + h1) * np.sin(latitude_geodetic_rad)
-    rr = np.sqrt(X**2.0 + Y**2.0 + Z**2.0)
+    X,Y,Z = pyTMD.spatial.to_cartesian(lon,lat,h=h1,a_axis=a_axis,flat=flat)
     #-- calculate geocentric latitude and convert to degrees
     latitude_geocentric = np.arctan(Z / np.sqrt(X**2.0 + Y**2.0))/dtr
 
@@ -509,8 +504,8 @@ def compute_OPT_icebridge_data(arg,METHOD=None,VERBOSE=False,MODE=0o775):
     iur,iun,iue,ilon,ilat = read_ocean_pole_tide(ocean_pole_tide_file)
 
     #-- pole tide files (mean and daily)
-    mean_pole_file = pyTMD.utilities.get_data_path(['data','mean-pole.tab'])
-    pole_tide_file = pyTMD.utilities.get_data_path(['data','finals.all'])
+    mean_pole_file = get_data_path(['data','mean-pole.tab'])
+    pole_tide_file = get_data_path(['data','finals.all'])
 
     #-- read IERS daily polar motion values
     EOP = read_iers_EOP(pole_tide_file)
@@ -608,7 +603,7 @@ def compute_OPT_icebridge_data(arg,METHOD=None,VERBOSE=False,MODE=0o775):
     fid.attrs['project'] = 'NASA_Operation_IceBridge'
     fid.attrs['processing_level'] = '4'
     fid.attrs['date_created'] = time.strftime('%Y-%m-%d',time.localtime())
-    #-- add attributes for input files
+    #-- add attributes for input file
     fid.attrs['elevation_file'] = os.path.basename(input_file)
     #-- add geospatial and temporal attributes
     fid.attrs['geospatial_lat_min'] = dinput['lat'].min()
@@ -623,8 +618,8 @@ def compute_OPT_icebridge_data(arg,METHOD=None,VERBOSE=False,MODE=0o775):
     #-- convert start/end time from MJD into Julian days
     JD_start = np.min(t) + 2400000.5
     JD_end = np.max(t) + 2400000.5
-    #-- convert to calendar date with convert_julian.py
-    cal = convert_julian(np.array([JD_start,JD_end]),ASTYPE=np.int)
+    #-- convert to calendar date
+    cal = pyTMD.time.convert_julian(np.array([JD_start,JD_end]),ASTYPE=int)
     #-- add attributes with measurement date start, end and duration
     args = (cal['hour'][0],cal['minute'][0],cal['second'][0])
     fid.attrs['RangeBeginningTime'] = '{0:02d}:{1:02d}:{2:02d}'.format(*args)
