@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 u"""
 compute_tidal_currents.py
-Written by Tyler Sutterley (11/2020)
+Written by Tyler Sutterley (03/2021)
 Calculates zonal and meridional tidal currents for an input file
 
 Uses OTIS format tidal solutions provided by Ohio State University and ESR
@@ -21,8 +21,10 @@ COMMAND LINE OPTIONS:
     -T X, --tide X: Tide model to use in correction
         CATS0201
         CATS2008
-        TPXO9-atlas-v2
         TPXO9-atlas
+        TPXO9-atlas-v2
+        TPXO9-atlas-v3
+        TPXO9-atlas-v4
         TPXO9.1
         TPXO8-atlas
         TPXO7.2
@@ -43,7 +45,7 @@ COMMAND LINE OPTIONS:
     -t X, --type X: input data type
         drift: drift buoys or satellite/airborne altimetry (time per data point)
         grid: spatial grids or images (single time for all data points)
-    --epoch X: Reference epoch of input time (default Modified Julian Day)
+    -e X, --epoch X: Reference epoch of input time (default Modified Julian Day)
         days since 1858-11-17T00:00:00
     -d X, --deltatime X: Input delta time for files without date information
         can be set to 0 to use exact calendar date from epoch
@@ -54,6 +56,7 @@ COMMAND LINE OPTIONS:
         linear
         nearest
         bilinear
+    -E X, --extrapolate X: Extrapolate with nearest-neighbors
     -V, --verbose: Verbose output of processing run
     -M X, --mode X: Permission mode of output file
 
@@ -87,9 +90,15 @@ PROGRAM DEPENDENCIES:
     read_tide_model.py: extract tidal harmonic constants from OTIS tide models
     read_netcdf_model.py: extract tidal harmonic constants from netcdf models
     read_FES_model.py: extract tidal harmonic constants from FES tide models
+    bilinear_interp.py: bilinear interpolation of data to coordinates
+    nearest_extrap.py: nearest-neighbor extrapolation of data to coordinates
     predict_tide_drift.py: predict tidal elevations using harmonic constants
 
 UPDATE HISTORY:
+    Updated 03/2021: added TPXO9-atlas-v4 in binary OTIS format
+        simplified netcdf inputs to be similar to binary OTIS read program
+    Updated 02/2021: replaced numpy bool to prevent deprecation warning
+    Updated 12/2020: added valid data extrapolation with nearest_extrap
     Updated 11/2020: added options to read from and write to geotiff image files
     Updated 10/2020: using argparse to set command line parameters
     Forked 09/2020 from compute_tidal_elevations.py
@@ -127,7 +136,8 @@ from pyTMD.read_FES_model import extract_FES_constants
 def compute_tidal_currents(tide_dir, input_file, output_file,
     TIDE_MODEL=None, FORMAT='csv', VARIABLES=['time','lat','lon','data'],
     HEADER=0, TYPE='drift', TIME_UNITS='days since 1858-11-17T00:00:00',
-    TIME=None, PROJECTION='4326', METHOD='spline', VERBOSE=False, MODE=0o775):
+    TIME=None, PROJECTION='4326', METHOD='spline', EXTRAPOLATE=False,
+    VERBOSE=False, MODE=0o775):
 
     #-- select between tide models
     if (TIDE_MODEL == 'CATS0201'):
@@ -147,7 +157,7 @@ def compute_tidal_currents(tide_dir, input_file, output_file,
         TYPES = ['u','v']
     elif (TIDE_MODEL == 'TPXO9-atlas'):
         model_directory = os.path.join(tide_dir,'TPXO9_atlas')
-        grid_file = 'grid_tpxo9_atlas.nc.gz'
+        grid_file = os.path.join(tide_dir,'grid_tpxo9_atlas.nc.gz')
         model_files = {}
         model_files['u'] = ['u_q1_tpxo9_atlas_30.nc.gz','u_o1_tpxo9_atlas_30.nc.gz',
             'u_p1_tpxo9_atlas_30.nc.gz','u_k1_tpxo9_atlas_30.nc.gz',
@@ -161,6 +171,9 @@ def compute_tidal_currents(tide_dir, input_file, output_file,
             'v_s2_tpxo9_atlas_30.nc.gz','v_k2_tpxo9_atlas_30.nc.gz',
             'v_m4_tpxo9_atlas_30.nc.gz','v_ms4_tpxo9_atlas_30.nc.gz',
             'v_mn4_tpxo9_atlas_30.nc.gz','v_2n2_tpxo9_atlas_30.nc.gz']
+        model_file = {}
+        for key,val in model_files.items():
+            model_file[key] = [os.path.join(model_directory,m) for m in val]
         reference = 'http://volkov.oce.orst.edu/tides/tpxo9_atlas.html'
         model_format = 'netcdf'
         TYPES = ['u','v']
@@ -168,7 +181,7 @@ def compute_tidal_currents(tide_dir, input_file, output_file,
         GZIP = True
     elif (TIDE_MODEL == 'TPXO9-atlas-v2'):
         model_directory = os.path.join(tide_dir,'TPXO9_atlas_v2')
-        grid_file = 'grid_tpxo9_atlas_30_v2.nc.gz'
+        grid_file = os.path.join(tide_dir,'grid_tpxo9_atlas_30_v2.nc.gz')
         model_files = {}
         model_files['u'] = ['u_q1_tpxo9_atlas_30_v2.nc.gz','u_o1_tpxo9_atlas_30_v2.nc.gz',
             'u_p1_tpxo9_atlas_30_v2.nc.gz','u_k1_tpxo9_atlas_30_v2.nc.gz',
@@ -182,11 +195,52 @@ def compute_tidal_currents(tide_dir, input_file, output_file,
             'v_s2_tpxo9_atlas_30_v2.nc.gz','v_k2_tpxo9_atlas_30_v2.nc.gz',
             'v_m4_tpxo9_atlas_30_v2.nc.gz','v_ms4_tpxo9_atlas_30_v2.nc.gz',
             'v_mn4_tpxo9_atlas_30_v2.nc.gz','v_2n2_tpxo9_atlas_30_v2.nc.gz']
+        model_file = {}
+        for key,val in model_files.items():
+            model_file[key] = [os.path.join(model_directory,m) for m in val]
         reference = 'https://www.tpxo.net/global/tpxo9-atlas'
         model_format = 'netcdf'
         TYPES = ['u','v']
         model_scale = 1.0/100.0
         GZIP = True
+    elif (TIDE_MODEL == 'TPXO9-atlas-v3'):
+        model_directory = os.path.join(tide_dir,'TPXO9_atlas_v3')
+        grid_file = os.path.join(tide_dir,'grid_tpxo9_atlas_30_v3.nc.gz')
+        model_files = {}
+        model_files['u'] = ['u_q1_tpxo9_atlas_30_v3.nc.gz','u_o1_tpxo9_atlas_30_v3.nc.gz',
+            'u_p1_tpxo9_atlas_30_v3.nc.gz','u_k1_tpxo9_atlas_30_v3.nc.gz',
+            'u_n2_tpxo9_atlas_30_v3.nc.gz','u_m2_tpxo9_atlas_30_v3.nc.gz',
+            'u_s2_tpxo9_atlas_30_v3.nc.gz','u_k2_tpxo9_atlas_30_v3.nc.gz',
+            'u_m4_tpxo9_atlas_30_v3.nc.gz','u_ms4_tpxo9_atlas_30_v3.nc.gz',
+            'u_mn4_tpxo9_atlas_30_v3.nc.gz','u_2n2_tpxo9_atlas_30_v3.nc.gz']
+        model_files['v'] = ['v_q1_tpxo9_atlas_30_v3.nc.gz','v_o1_tpxo9_atlas_30_v3.nc.gz',
+            'v_p1_tpxo9_atlas_30_v3.nc.gz','v_k1_tpxo9_atlas_30_v3.nc.gz',
+            'v_n2_tpxo9_atlas_30_v3.nc.gz','v_m2_tpxo9_atlas_30_v3.nc.gz',
+            'v_s2_tpxo9_atlas_30_v3.nc.gz','v_k2_tpxo9_atlas_30_v3.nc.gz',
+            'v_m4_tpxo9_atlas_30_v3.nc.gz','v_ms4_tpxo9_atlas_30_v3.nc.gz',
+            'v_mn4_tpxo9_atlas_30_v3.nc.gz','v_2n2_tpxo9_atlas_30_v3.nc.gz']
+        model_file = {}
+        for key,val in model_files.items():
+            model_file[key] = [os.path.join(model_directory,m) for m in val]
+        reference = 'https://www.tpxo.net/global/tpxo9-atlas'
+        model_format = 'netcdf'
+        TYPES = ['u','v']
+        model_scale = 1.0/100.0
+        GZIP = True
+    elif (TIDE_MODEL == 'TPXO9-atlas-v4'):
+        model_directory = os.path.join(tide_dir,'TPXO9_atlas_v4')
+        grid_file = os.path.join(tide_dir,'grid_tpxo9_atlas_30_v4')
+        model_files = ['u_q1_tpxo9_atlas_30_v4','u_o1_tpxo9_atlas_30_v4',
+            'u_p1_tpxo9_atlas_30_v4','u_k1_tpxo9_atlas_30_v4',
+            'u_n2_tpxo9_atlas_30_v4','u_m2_tpxo9_atlas_30_v4',
+            'u_s2_tpxo9_atlas_30_v4','u_k2_tpxo9_atlas_30_v4',
+            'u_m4_tpxo9_atlas_30_v4','u_ms4_tpxo9_atlas_30_v4',
+            'u_mn4_tpxo9_atlas_30_v4','u_2n2_tpxo9_atlas_30_v4']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
+        reference = 'https://www.tpxo.net/global/tpxo9-atlas'
+        model_format = 'OTIS'
+        EPSG = '4326'
+        TYPES = ['u','v']
     elif (TIDE_MODEL == 'TPXO9.1'):
         grid_file = os.path.join(tide_dir,'TPXO9.1','DATA','grid_tpxo9')
         model_file = os.path.join(tide_dir,'TPXO9.1','DATA','u_tpxo9.v1')
@@ -243,14 +297,18 @@ def compute_tidal_currents(tide_dir, input_file, output_file,
             'mu2.nc.gz','n2.nc.gz','n4.nc.gz','nu2.nc.gz','o1.nc.gz','p1.nc.gz',
             'q1.nc.gz','r2.nc.gz','s1.nc.gz','s2.nc.gz','s4.nc.gz','sa.nc.gz',
             'ssa.nc.gz','t2.nc.gz']
+        model_file = {}
+        for key,val in model_directory.items():
+            model_file[key] = [os.path.join(val,m) for m in model_files]
         c = ['2n2','eps2','j1','k1','k2','l2','lambda2','m2','m3','m4','m6',
             'm8','mf','mks2','mm','mn4','ms4','msf','msqm','mtm','mu2','n2',
             'n4','nu2','o1','p1','q1','r2','s1','s2','s4','sa','ssa','t2']
-        reference = ('https://www.aviso.altimetry.fr/data/products/'
+        reference = ('https://www.aviso.altimetry.fr/en/data/products'
             'auxiliary-products/global-tide-fes.html')
         model_format = 'FES'
         TYPES = ['u','v']
-        model_scale = 1.0/100.0
+        model_scale = 1.0
+        GZIP = True
 
     #-- invalid value
     fill_value = -9999.0
@@ -335,6 +393,8 @@ def compute_tidal_currents(tide_dir, input_file, output_file,
         epoch1=epoch1, epoch2=(1992,1,1,0,0,0), scale=1.0/86400.0)
     #-- number of time points
     nt = len(tide_time)
+    #-- delta time (TT - UT1) file
+    delta_file = get_data_path(['data','merged_deltat.data'])
 
     #-- python dictionary with output data
     output = {'time':tide_time,'lon':lon,'lat':lat}
@@ -344,19 +404,18 @@ def compute_tidal_currents(tide_dir, input_file, output_file,
         if model_format in ('OTIS','ATLAS'):
             amp,ph,D,c = extract_tidal_constants(lon.flatten(), lat.flatten(),
                 grid_file, model_file, EPSG, TYPE=t, METHOD=METHOD,
-                GRID=model_format)
+                EXTRAPOLATE=EXTRAPOLATE, GRID=model_format)
             deltat = np.zeros((nt))
         elif (model_format == 'netcdf'):
             amp,ph,D,c = extract_netcdf_constants(lon.flatten(), lat.flatten(),
-                model_directory, grid_file, model_files[t], TYPE=t,
-                METHOD=METHOD, SCALE=model_scale, GZIP=GZIP)
+                grid_file, model_file[t], TYPE=t, METHOD=METHOD,
+                EXTRAPOLATE=EXTRAPOLATE, SCALE=model_scale, GZIP=GZIP)
             deltat = np.zeros((nt))
         elif (model_format == 'FES'):
             amp,ph = extract_FES_constants(lon.flatten(), lat.flatten(),
-                model_directory[t], model_files, TYPE=t, VERSION=TIDE_MODEL,
-                METHOD=METHOD, SCALE=model_scale)
+                model_file[t], TYPE=t, VERSION=TIDE_MODEL, METHOD=METHOD,
+                EXTRAPOLATE=EXTRAPOLATE, SCALE=model_scale, GZIP=GZIP)
             #-- interpolate delta times from calendar dates to tide time
-            delta_file = get_data_path(['data','merged_deltat.data'])
             deltat = calc_delta_time(delta_file, tide_time)
 
         #-- calculate complex phase in radians for Euler's
@@ -367,7 +426,7 @@ def compute_tidal_currents(tide_dir, input_file, output_file,
         #-- predict tidal currents at time and infer minor corrections
         if (TYPE == 'grid'):
             output[t] = np.ma.zeros((ny,nx,nt),fill_value=fill_value)
-            output[t].mask = np.zeros((ny,nx,nt),dtype=np.bool)
+            output[t].mask = np.zeros((ny,nx,nt),dtype=bool)
             for i in range(nt):
                 TIDE = predict_tide(tide_time[i], hc, c,
                     DELTAT=deltat[i], CORRECTIONS=model_format)
@@ -428,8 +487,8 @@ def main():
         help='Working data directory')
     #-- tide model to use
     model_choices = ('CATS0201','CATS2008','TPXO9-atlas','TPXO9-atlas-v2',
-        'TPXO9.1','TPXO8-atlas','TPXO7.2','AODTM-5','AOTIM-5','AOTIM-5-2018',
-        'FES2014')
+        'TPXO9-atlas-v3','TPXO9-atlas-v4','TPXO9.1','TPXO8-atlas','TPXO7.2',
+        'AODTM-5','AOTIM-5','AOTIM-5-2018','FES2014')
     parser.add_argument('--tide','-T',
         metavar='TIDE', type=str, default='CATS2008',
         choices=model_choices,
@@ -455,7 +514,7 @@ def main():
         help='Input data type')
     #-- time epoch (default Modified Julian Days)
     #-- in form "time-units since yyyy-mm-dd hh:mm:ss"
-    parser.add_argument('--epoch','-E',
+    parser.add_argument('--epoch','-e',
         type=str, default='days since 1858-11-17T00:00:00',
         help='Reference epoch of input time')
     #-- input delta time for files without date information
@@ -471,6 +530,10 @@ def main():
         metavar='METHOD', type=str, default='spline',
         choices=('spline','linear','nearest','bilinear'),
         help='Spatial interpolation method')
+    #-- extrapolate with nearest-neighbors
+    parser.add_argument('--extrapolate','-E',
+        default=False, action='store_true',
+        help='Extrapolate with nearest-neighbors')
     #-- verbose output of processing run
     #-- print information about each input and output file
     parser.add_argument('--verbose','-V',
@@ -493,7 +556,8 @@ def main():
         FORMAT=args.format, TIDE_MODEL=args.tide, VARIABLES=args.variables,
         HEADER=args.header, TYPE=args.type, TIME_UNITS=args.epoch,
         TIME=args.deltatime, PROJECTION=args.projection,
-        METHOD=args.interpolate, VERBOSE=args.verbose, MODE=args.mode)
+        METHOD=args.interpolate, EXTRAPOLATE=args.extrapolate,
+        VERBOSE=args.verbose, MODE=args.mode)
 
 #-- run main program
 if __name__ == '__main__':
