@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 u"""
 compute_tides_ICESat2_ATL07.py
-Written by Tyler Sutterley (11/2020)
+Written by Tyler Sutterley (04/2021)
 Calculates tidal elevations for correcting ICESat-2 sea ice height data
 
 Uses OTIS format tidal solutions provided by Ohio State University and ESR
@@ -20,6 +20,7 @@ COMMAND LINE OPTIONS:
         TPXO9-atlas
         TPXO9-atlas-v2
         TPXO9-atlas-v3
+        TPXO9-atlas-v4
         TPXO9.1
         TPXO8-atlas
         TPXO7.2
@@ -40,6 +41,7 @@ COMMAND LINE OPTIONS:
         linear
         nearest
         bilinear
+    -E X, --extrapolate X: Extrapolate with nearest-neighbors
     -M X, --mode X: Permission mode of directories and files created
     -V, --verbose: Output information about each created file
 
@@ -58,20 +60,28 @@ PROGRAM DEPENDENCIES:
     read_ICESat2_ATL07.py: reads ICESat-2 sea ice height data files
     time.py: utilities for calculating time operations
     utilities: download and management utilities for syncing files
-    convert_julian.py: returns the calendar date and time given a Julian date
     calc_astrol_longitudes.py: computes the basic astronomical mean longitudes
     calc_delta_time.py: calculates difference between universal and dynamic time
     convert_ll_xy.py: convert lat/lon points to and from projected coordinates
     infer_minor_corrections.py: return corrections for minor constituents
     load_constituent.py: loads parameters for a given tidal constituent
     load_nodal_corrections.py: load the nodal corrections for tidal constituents
-    predict_tide_drift.py: predict tidal elevations using harmonic constants
     read_tide_model.py: extract tidal harmonic constants from OTIS tide models
     read_netcdf_model.py: extract tidal harmonic constants from netcdf models
     read_GOT_model.py: extract tidal harmonic constants from GSFC GOT models
     read_FES_model.py: extract tidal harmonic constants from FES tide models
+    bilinear_interp.py: bilinear interpolation of data to coordinates
+    nearest_extrap.py: nearest-neighbor extrapolation of data to coordinates
+    predict_tide_drift.py: predict tidal elevations using harmonic constants
 
 UPDATE HISTORY:
+    Updated 04/2021: can use a generically named ATL07 file as input
+    Updated 03/2021: added TPXO9-atlas-v4 in binary OTIS format
+        simplified netcdf inputs to be similar to binary OTIS read program
+        replaced numpy bool/int to prevent deprecation warnings
+    Updated 12/2020: H5py deprecation warning change to use make_scale
+        added valid data extrapolation with nearest_extrap
+        merged time conversion routines into module
     Updated 11/2020: added model constituents from TPXO9-atlas-v3
     Updated 10/2020: using argparse to set command line parameters
     Updated 08/2020: using builtin time operations.  python3 regular expressions
@@ -87,7 +97,7 @@ UPDATE HISTORY:
     Updated 09/2019: using date functions paralleling public repository
         add option for TPXO9-atlas.  add OTIS netcdf tide option
     Updated 05/2019: check if beam exists in a try except else clause
-    Updated 04/2019: check if subsetted beam contains land ice data
+    Updated 04/2019: check if subsetted beam contains sea ice data
     Written 04/2019
 """
 from __future__ import print_function
@@ -101,7 +111,6 @@ import datetime
 import numpy as np
 import pyTMD.time
 import pyTMD.utilities
-from pyTMD.convert_julian import convert_julian
 from pyTMD.calc_delta_time import calc_delta_time
 from pyTMD.read_tide_model import extract_tidal_constants
 from pyTMD.read_netcdf_model import extract_netcdf_constants
@@ -113,8 +122,8 @@ from icesat2_toolkit.read_ICESat2_ATL07 import read_HDF5_ATL07
 
 #-- PURPOSE: read ICESat-2 sea ice height (ATL07) from NSIDC
 #-- compute tides at points and times using tidal model driver algorithms
-def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
-    VERBOSE=False, MODE=0o775):
+def compute_tides_ICESat2(tide_dir, INPUT_FILE, TIDE_MODEL=None, METHOD='spline',
+    EXTRAPOLATE=False, VERBOSE=False, MODE=0o775):
     #-- select between tide models
     if (TIDE_MODEL == 'CATS0201'):
         grid_file = os.path.join(tide_dir,'cats0201_tmd','grid_CATS')
@@ -154,13 +163,14 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
         TYPE = 'z'
     elif (TIDE_MODEL == 'TPXO9-atlas'):
         model_directory = os.path.join(tide_dir,'TPXO9_atlas')
-        grid_file = 'grid_tpxo9_atlas.nc.gz'
+        grid_file = os.path.join(model_directory,'grid_tpxo9_atlas.nc.gz')
         model_files = ['h_q1_tpxo9_atlas_30.nc.gz','h_o1_tpxo9_atlas_30.nc.gz',
             'h_p1_tpxo9_atlas_30.nc.gz','h_k1_tpxo9_atlas_30.nc.gz',
             'h_n2_tpxo9_atlas_30.nc.gz','h_m2_tpxo9_atlas_30.nc.gz',
             'h_s2_tpxo9_atlas_30.nc.gz','h_k2_tpxo9_atlas_30.nc.gz',
             'h_m4_tpxo9_atlas_30.nc.gz','h_ms4_tpxo9_atlas_30.nc.gz',
             'h_mn4_tpxo9_atlas_30.nc.gz','h_2n2_tpxo9_atlas_30.nc.gz']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         reference = 'http://volkov.oce.orst.edu/tides/tpxo9_atlas.html'
         variable = 'height_segment_ocean'
         long_name = "Ocean Tide"
@@ -170,15 +180,17 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
         model_format = 'netcdf'
         TYPE = 'z'
         SCALE = 1.0/1000.0
+        GZIP = True
     elif (TIDE_MODEL == 'TPXO9-atlas-v2'):
         model_directory = os.path.join(tide_dir,'TPXO9_atlas_v2')
-        grid_file = 'grid_tpxo9_atlas_30_v2.nc.gz'
+        grid_file = os.path.join(model_directory,'grid_tpxo9_atlas_30_v2.nc.gz')
         model_files = ['h_q1_tpxo9_atlas_30_v2.nc.gz','h_o1_tpxo9_atlas_30_v2.nc.gz',
             'h_p1_tpxo9_atlas_30_v2.nc.gz','h_k1_tpxo9_atlas_30_v2.nc.gz',
             'h_n2_tpxo9_atlas_30_v2.nc.gz','h_m2_tpxo9_atlas_30_v2.nc.gz',
             'h_s2_tpxo9_atlas_30_v2.nc.gz','h_k2_tpxo9_atlas_30_v2.nc.gz',
             'h_m4_tpxo9_atlas_30_v2.nc.gz','h_ms4_tpxo9_atlas_30_v2.nc.gz',
             'h_mn4_tpxo9_atlas_30_v2.nc.gz','h_2n2_tpxo9_atlas_30_v2.nc.gz']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         reference = 'https://www.tpxo.net/global/tpxo9-atlas'
         variable = 'height_segment_ocean'
         long_name = "Ocean Tide"
@@ -188,9 +200,10 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
         model_format = 'netcdf'
         TYPE = 'z'
         SCALE = 1.0/1000.0
+        GZIP = True
     elif (TIDE_MODEL == 'TPXO9-atlas-v3'):
         model_directory = os.path.join(tide_dir,'TPXO9_atlas_v3')
-        grid_file = 'grid_tpxo9_atlas_30_v3.nc.gz'
+        grid_file = os.path.join(model_directory,'grid_tpxo9_atlas_30_v3.nc.gz')
         model_files = ['h_q1_tpxo9_atlas_30_v3.nc.gz','h_o1_tpxo9_atlas_30_v3.nc.gz',
             'h_p1_tpxo9_atlas_30_v3.nc.gz','h_k1_tpxo9_atlas_30_v3.nc.gz',
             'h_n2_tpxo9_atlas_30_v3.nc.gz','h_m2_tpxo9_atlas_30_v3.nc.gz',
@@ -198,6 +211,7 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
             'h_m4_tpxo9_atlas_30_v3.nc.gz','h_ms4_tpxo9_atlas_30_v3.nc.gz',
             'h_mn4_tpxo9_atlas_30_v3.nc.gz','h_2n2_tpxo9_atlas_30_v3.nc.gz',
             'h_mf_tpxo9_atlas_30_v3.nc.gz','h_mm_tpxo9_atlas_30_v3.nc.gz']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         reference = 'https://www.tpxo.net/global/tpxo9-atlas'
         variable = 'tide_ocean'
         long_name = "Ocean Tide"
@@ -207,6 +221,27 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
         model_format = 'netcdf'
         TYPE = 'z'
         SCALE = 1.0/1000.0
+        GZIP = True
+    elif (TIDE_MODEL == 'TPXO9-atlas-v4'):
+        model_directory = os.path.join(tide_dir,'TPXO9_atlas_v4')
+        grid_file = os.path.join(model_directory,'grid_tpxo9_atlas_30_v4')
+        model_files = ['h_q1_tpxo9_atlas_30_v4','h_o1_tpxo9_atlas_30_v4',
+            'h_p1_tpxo9_atlas_30_v4','h_k1_tpxo9_atlas_30_v4',
+            'h_n2_tpxo9_atlas_30_v4','h_m2_tpxo9_atlas_30_v4',
+            'h_s2_tpxo9_atlas_30_v4','h_k2_tpxo9_atlas_30_v4',
+            'h_m4_tpxo9_atlas_30_v4','h_ms4_tpxo9_atlas_30_v4',
+            'h_mn4_tpxo9_atlas_30_v4','h_2n2_tpxo9_atlas_30_v4',
+            'h_mf_tpxo9_atlas_30_v4','h_mm_tpxo9_atlas_30_v4']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
+        reference = 'https://www.tpxo.net/global/tpxo9-atlas'
+        variable = 'height_segment_ocean'
+        long_name = "Ocean Tide"
+        description = ("Ocean Tides including diurnal and semi-diurnal "
+            "(harmonic analysis), and longer period tides (dynamic and "
+            "self-consistent equilibrium).")
+        model_format = 'OTIS'
+        EPSG = '4326'
+        TYPE = 'z'
     elif (TIDE_MODEL == 'TPXO9.1'):
         grid_file = os.path.join(tide_dir,'TPXO9.1','DATA','grid_tpxo9')
         model_file = os.path.join(tide_dir,'TPXO9.1','DATA','h_tpxo9.v1')
@@ -296,7 +331,7 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
         model_directory = os.path.join(tide_dir,'GOT4.7','grids_oceantide')
         model_files = ['q1.d.gz','o1.d.gz','p1.d.gz','k1.d.gz','n2.d.gz',
             'm2.d.gz','s2.d.gz','k2.d.gz','s1.d.gz','m4.d.gz']
-        c = ['q1','o1','p1','k1','n2','m2','s2','k2','s1','m4']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         reference = ('https://denali.gsfc.nasa.gov/personal_pages/ray/'
             'MiscPubs/19990089548_1999150788.pdf')
         variable = 'height_segment_ocean'
@@ -306,12 +341,13 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
             "self-consistent equilibrium).")
         model_format = 'GOT'
         SCALE = 1.0/100.0
+        GZIP = True
     elif (TIDE_MODEL == 'GOT4.7_load'):
         model_directory = os.path.join(tide_dir,'GOT4.7','grids_loadtide')
         model_files = ['q1load.d.gz','o1load.d.gz','p1load.d.gz','k1load.d.gz',
             'n2load.d.gz','m2load.d.gz','s2load.d.gz','k2load.d.gz',
             's1load.d.gz','m4load.d.gz']
-        c = ['q1','o1','p1','k1','n2','m2','s2','k2','s1','m4']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         reference = ('https://denali.gsfc.nasa.gov/personal_pages/ray/'
             'MiscPubs/19990089548_1999150788.pdf')
         variable = 'height_segment_load'
@@ -319,11 +355,12 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
         description = "Local displacement due to Ocean Loading (-6 to 0 cm)"
         model_format = 'GOT'
         SCALE = 1.0/1000.0
+        GZIP = True
     elif (TIDE_MODEL == 'GOT4.8'):
         model_directory = os.path.join(tide_dir,'got4.8','grids_oceantide')
         model_files = ['q1.d.gz','o1.d.gz','p1.d.gz','k1.d.gz','n2.d.gz',
             'm2.d.gz','s2.d.gz','k2.d.gz','s1.d.gz','m4.d.gz']
-        c = ['q1','o1','p1','k1','n2','m2','s2','k2','s1','m4']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         reference = ('https://denali.gsfc.nasa.gov/personal_pages/ray/'
             'MiscPubs/19990089548_1999150788.pdf')
         variable = 'height_segment_ocean'
@@ -333,12 +370,13 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
             "self-consistent equilibrium).")
         model_format = 'GOT'
         SCALE = 1.0/100.0
+        GZIP = True
     elif (TIDE_MODEL == 'GOT4.8_load'):
         model_directory = os.path.join(tide_dir,'got4.8','grids_loadtide')
         model_files = ['q1load.d.gz','o1load.d.gz','p1load.d.gz','k1load.d.gz',
             'n2load.d.gz','m2load.d.gz','s2load.d.gz','k2load.d.gz',
             's1load.d.gz','m4load.d.gz']
-        c = ['q1','o1','p1','k1','n2','m2','s2','k2','s1','m4']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         reference = ('https://denali.gsfc.nasa.gov/personal_pages/ray/'
             'MiscPubs/19990089548_1999150788.pdf')
         variable = 'height_segment_load'
@@ -346,11 +384,12 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
         description = "Local displacement due to Ocean Loading (-6 to 0 cm)"
         model_format = 'GOT'
         SCALE = 1.0/1000.0
+        GZIP = True
     elif (TIDE_MODEL == 'GOT4.10'):
         model_directory = os.path.join(tide_dir,'GOT4.10c','grids_oceantide')
         model_files = ['q1.d.gz','o1.d.gz','p1.d.gz','k1.d.gz','n2.d.gz',
             'm2.d.gz','s2.d.gz','k2.d.gz','s1.d.gz','m4.d.gz']
-        c = ['q1','o1','p1','k1','n2','m2','s2','k2','s1','m4']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         reference = ('https://denali.gsfc.nasa.gov/personal_pages/ray/'
             'MiscPubs/19990089548_1999150788.pdf')
         variable = 'height_segment_ocean'
@@ -360,12 +399,13 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
             "self-consistent equilibrium).")
         model_format = 'GOT'
         SCALE = 1.0/100.0
+        GZIP = True
     elif (TIDE_MODEL == 'GOT4.10_load'):
         model_directory = os.path.join(tide_dir,'GOT4.10c','grids_loadtide')
         model_files = ['q1load.d.gz','o1load.d.gz','p1load.d.gz','k1load.d.gz',
             'n2load.d.gz','m2load.d.gz','s2load.d.gz','k2load.d.gz',
             's1load.d.gz','m4load.d.gz']
-        c = ['q1','o1','p1','k1','n2','m2','s2','k2','s1','m4']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         reference = ('https://denali.gsfc.nasa.gov/personal_pages/ray/'
             'MiscPubs/19990089548_1999150788.pdf')
         variable = 'height_segment_load'
@@ -373,6 +413,7 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
         description = "Local displacement due to Ocean Loading (-6 to 0 cm)"
         model_format = 'GOT'
         SCALE = 1.0/1000.0
+        GZIP = True
     elif (TIDE_MODEL == 'FES2014'):
         model_directory = os.path.join(tide_dir,'fes2014','ocean_tide')
         model_files = ['2n2.nc.gz','eps2.nc.gz','j1.nc.gz','k1.nc.gz',
@@ -382,10 +423,11 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
             'mu2.nc.gz','n2.nc.gz','n4.nc.gz','nu2.nc.gz','o1.nc.gz','p1.nc.gz',
             'q1.nc.gz','r2.nc.gz','s1.nc.gz','s2.nc.gz','s4.nc.gz','sa.nc.gz',
             'ssa.nc.gz','t2.nc.gz']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         c = ['2n2','eps2','j1','k1','k2','l2','lambda2','m2','m3','m4','m6',
             'm8','mf','mks2','mm','mn4','ms4','msf','msqm','mtm','mu2','n2',
             'n4','nu2','o1','p1','q1','r2','s1','s2','s4','sa','ssa','t2']
-        reference = ('https://www.aviso.altimetry.fr/data/products/'
+        reference = ('https://www.aviso.altimetry.fr/en/data/products'
             'auxiliary-products/global-tide-fes.html')
         variable = 'height_segment_ocean'
         long_name = "Ocean Tide"
@@ -395,6 +437,7 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
         model_format = 'FES'
         TYPE = 'z'
         SCALE = 1.0/100.0
+        GZIP = True
     elif (TIDE_MODEL == 'FES2014_load'):
         model_directory = os.path.join(tide_dir,'fes2014','load_tide')
         model_files = ['2n2.nc.gz','eps2.nc.gz','j1.nc.gz','k1.nc.gz',
@@ -404,10 +447,11 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
             'mu2.nc.gz','n2.nc.gz','n4.nc.gz','nu2.nc.gz','o1.nc.gz','p1.nc.gz',
             'q1.nc.gz','r2.nc.gz','s1.nc.gz','s2.nc.gz','s4.nc.gz','sa.nc.gz',
             'ssa.nc.gz','t2.nc.gz']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         c = ['2n2','eps2','j1','k1','k2','l2','lambda2','m2','m3','m4','m6',
             'm8','mf','mks2','mm','mn4','ms4','msf','msqm','mtm','mu2','n2',
             'n4','nu2','o1','p1','q1','r2','s1','s2','s4','sa','ssa','t2']
-        reference = ('https://www.aviso.altimetry.fr/data/products/'
+        reference = ('https://www.aviso.altimetry.fr/en/data/products'
             'auxiliary-products/global-tide-fes.html')
         variable = 'height_segment_load'
         long_name = "Load Tide"
@@ -415,24 +459,39 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
         model_format = 'FES'
         TYPE = 'z'
         SCALE = 1.0/100.0
+        GZIP = True
 
-    #-- read data from FILE
-    print('{0} -->'.format(os.path.basename(FILE))) if VERBOSE else None
-    IS2_atl07_mds,IS2_atl07_attrs,IS2_atl07_beams = read_HDF5_ATL07(FILE,
+    #-- read data from input file
+    print('{0} -->'.format(os.path.basename(INPUT_FILE))) if VERBOSE else None
+    IS2_atl07_mds,IS2_atl07_attrs,IS2_atl07_beams = read_HDF5_ATL07(INPUT_FILE,
         ATTRIBUTES=True)
-    DIRECTORY = os.path.dirname(FILE)
+    DIRECTORY = os.path.dirname(INPUT_FILE)
     #-- extract parameters from ICESat-2 ATLAS HDF5 sea ice file name
     rx = re.compile(r'(processed_)?(ATL\d{2})-(\d{2})_(\d{4})(\d{2})(\d{2})'
-        '(\d{2})(\d{2})(\d{2})_(\d{4})(\d{2})(\d{2})_(\d{3})_(\d{2})(.*?).h5$')
-    SUB,PRD,HEM,YY,MM,DD,HH,MN,SS,TRK,CYCL,SN,RL,VERS,AUX=rx.findall(FILE).pop()
+        r'(\d{2})(\d{2})(\d{2})_(\d{4})(\d{2})(\d{2})_(\d{3})_(\d{2})(.*?).h5$')
+    try:
+        SUB,PRD,HEM,YY,MM,DD,HH,MN,SS,TRK,CYCL,SN,RL,VERS,AUX=rx.findall(INPUT_FILE).pop()
+    except:
+        #-- output tide HDF5 file (generic)
+        fileBasename,fileExtension = os.path.splitext(INPUT_FILE)
+        args = (fileBasename,TIDE_MODEL,fileExtension)
+        OUTPUT_FILE = '{0}_{1}_TIDES{2}'.format(*args)
+    else:
+        #-- output tide HDF5 file for ASAS/NSIDC granules
+        args = (PRD,HEM,TIDE_MODEL,YY,MM,DD,HH,MN,SS,TRK,CYCL,SN,RL,VERS,AUX)
+        ff = '{0}-{1}_{2}_TIDES_{3}{4}{5}{6}{7}{8}_{9}{10}{11}_{12}_{13}{14}.h5'
+        OUTPUT_FILE = ff.format(*args)
 
     #-- number of GPS seconds between the GPS epoch
     #-- and ATLAS Standard Data Product (SDP) epoch
     atlas_sdp_gps_epoch = IS2_atl07_mds['ancillary_data']['atlas_sdp_gps_epoch']
+    #-- delta time (TT - UT1) file
+    delta_file = pyTMD.utilities.get_data_path(['data','merged_deltat.data'])
 
     #-- copy variables for outputting to HDF5 file
     IS2_atl07_tide = {}
     IS2_atl07_fill = {}
+    IS2_atl07_dims = {}
     IS2_atl07_tide_attrs = {}
     #-- number of GPS seconds between the GPS epoch (1980-01-06T00:00:00Z UTC)
     #-- and ATLAS Standard Data Product (SDP) epoch (2018-01-01T00:00:00Z UTC)
@@ -452,6 +511,7 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
         #-- output data dictionaries for beam
         IS2_atl07_tide[gtx] = dict(sea_ice_segments={})
         IS2_atl07_fill[gtx] = dict(sea_ice_segments={})
+        IS2_atl07_dims[gtx] = dict(sea_ice_segments={})
         IS2_atl07_tide_attrs[gtx] = dict(sea_ice_segments={})
 
         #-- number of segments
@@ -467,25 +527,24 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
         if model_format in ('OTIS','ATLAS'):
             amp,ph,D,c = extract_tidal_constants(val['longitude'],
                 val['latitude'], grid_file, model_file, EPSG, TYPE=TYPE,
-                METHOD=METHOD, GRID=model_format)
+                METHOD=METHOD, EXTRAPOLATE=EXTRAPOLATE, GRID=model_format)
             deltat = np.zeros_like(tide_time)
         elif (model_format == 'netcdf'):
             amp,ph,D,c = extract_netcdf_constants(val['longitude'],
-                val['latitude'], model_directory, grid_file,
-                model_files, TYPE=TYPE, METHOD=METHOD, SCALE=SCALE)
+                val['latitude'], grid_file, model_file, TYPE=TYPE, METHOD=METHOD,
+                EXTRAPOLATE=EXTRAPOLATE, SCALE=SCALE, GZIP=GZIP)
             deltat = np.zeros_like(tide_time)
         elif (model_format == 'GOT'):
-            amp,ph = extract_GOT_constants(val['longitude'], val['latitude'],
-                model_directory, model_files,METHOD=METHOD, SCALE=SCALE)
+            amp,ph,c = extract_GOT_constants(val['longitude'], val['latitude'],
+                model_file, METHOD=METHOD, EXTRAPOLATE=EXTRAPOLATE, SCALE=SCALE,
+                GZIP=GZIP)
             #-- interpolate delta times from calendar dates to tide time
-            delta_file = pyTMD.utilities.get_data_path(['data','merged_deltat.data'])
             deltat = calc_delta_time(delta_file, tide_time)
         elif (model_format == 'FES'):
             amp,ph = extract_FES_constants(val['longitude'], val['latitude'],
-                model_directory, model_files, TYPE=TYPE, VERSION=TIDE_MODEL,
-                METHOD=METHOD, SCALE=SCALE)
+                model_file, TYPE=TYPE, VERSION=TIDE_MODEL, METHOD=METHOD,
+                EXTRAPOLATE=EXTRAPOLATE, SCALE=SCALE, GZIP=GZIP)
             #-- interpolate delta times from calendar dates to tide time
-            delta_file = pyTMD.utilities.get_data_path(['data','merged_deltat.data'])
             deltat = calc_delta_time(delta_file, tide_time)
 
         #-- calculate complex phase in radians for Euler's
@@ -524,6 +583,7 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
         #-- delta time
         IS2_atl07_tide[gtx]['sea_ice_segments']['delta_time'] = val['delta_time'].copy()
         IS2_atl07_fill[gtx]['sea_ice_segments']['delta_time'] = None
+        IS2_atl07_dims[gtx]['sea_ice_segments']['delta_time'] = None
         IS2_atl07_tide_attrs[gtx]['sea_ice_segments']['delta_time'] = {}
         IS2_atl07_tide_attrs[gtx]['sea_ice_segments']['delta_time']['units'] = "seconds since 2018-01-01"
         IS2_atl07_tide_attrs[gtx]['sea_ice_segments']['delta_time']['long_name'] = "Elapsed GPS seconds"
@@ -538,10 +598,10 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
             "parameters, the time in gps_seconds relative to the GPS epoch can be computed.")
         IS2_atl07_tide_attrs[gtx]['sea_ice_segments']['delta_time']['coordinates'] = \
             "height_segment_id latitude longitude"
-
         #-- latitude
-        IS2_atl07_tide[gtx]['sea_ice_segments']['latitude'] = val['latitude']
+        IS2_atl07_tide[gtx]['sea_ice_segments']['latitude'] = val['latitude'].copy()
         IS2_atl07_fill[gtx]['sea_ice_segments']['latitude'] = None
+        IS2_atl07_dims[gtx]['sea_ice_segments']['latitude'] = ['delta_time']
         IS2_atl07_tide_attrs[gtx]['sea_ice_segments']['latitude'] = {}
         IS2_atl07_tide_attrs[gtx]['sea_ice_segments']['latitude']['units'] = "degrees_north"
         IS2_atl07_tide_attrs[gtx]['sea_ice_segments']['latitude']['contentType'] = "physicalMeasurement"
@@ -556,6 +616,7 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
         #-- longitude
         IS2_atl07_tide[gtx]['sea_ice_segments']['longitude'] = val['longitude'].copy()
         IS2_atl07_fill[gtx]['sea_ice_segments']['longitude'] = None
+        IS2_atl07_dims[gtx]['sea_ice_segments']['longitude'] = ['delta_time']
         IS2_atl07_tide_attrs[gtx]['sea_ice_segments']['longitude'] = {}
         IS2_atl07_tide_attrs[gtx]['sea_ice_segments']['longitude']['units'] = "degrees_east"
         IS2_atl07_tide_attrs[gtx]['sea_ice_segments']['longitude']['contentType'] = "physicalMeasurement"
@@ -570,6 +631,7 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
         #-- segment ID
         IS2_atl07_tide[gtx]['sea_ice_segments']['height_segment_id'] = val['height_segment_id']
         IS2_atl07_fill[gtx]['sea_ice_segments']['height_segment_id'] = None
+        IS2_atl07_dims[gtx]['sea_ice_segments']['height_segment_id'] = ['delta_time']
         IS2_atl07_tide_attrs[gtx]['sea_ice_segments']['height_segment_id'] = {}
         IS2_atl07_tide_attrs[gtx]['sea_ice_segments']['height_segment_id']['units'] = "1"
         IS2_atl07_tide_attrs[gtx]['sea_ice_segments']['height_segment_id']['contentType'] = "referenceInformation"
@@ -582,6 +644,7 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
         #-- geolocation segment beginning
         IS2_atl07_tide[gtx]['sea_ice_segments']['geoseg_beg'] = val['geoseg_beg'].copy()
         IS2_atl07_fill[gtx]['sea_ice_segments']['geoseg_beg'] = None
+        IS2_atl07_dims[gtx]['sea_ice_segments']['geoseg_beg'] = ['delta_time']
         IS2_atl07_tide_attrs[gtx]['sea_ice_segments']['geoseg_beg'] = {}
         IS2_atl07_tide_attrs[gtx]['sea_ice_segments']['geoseg_beg']['units'] = "1"
         IS2_atl07_tide_attrs[gtx]['sea_ice_segments']['geoseg_beg']['contentType'] = "referenceInformation"
@@ -593,6 +656,7 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
         #-- geolocation segment ending
         IS2_atl07_tide[gtx]['sea_ice_segments']['geoseg_end'] = val['geoseg_end'].copy()
         IS2_atl07_fill[gtx]['sea_ice_segments']['geoseg_end'] = None
+        IS2_atl07_dims[gtx]['sea_ice_segments']['geoseg_end'] = ['delta_time']
         IS2_atl07_tide_attrs[gtx]['sea_ice_segments']['geoseg_end'] = {}
         IS2_atl07_tide_attrs[gtx]['sea_ice_segments']['geoseg_end']['units'] = "1"
         IS2_atl07_tide_attrs[gtx]['sea_ice_segments']['geoseg_end']['contentType'] = "referenceInformation"
@@ -604,6 +668,7 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
         #-- along track distance
         IS2_atl07_tide[gtx]['sea_ice_segments']['seg_dist_x'] = val['seg_dist_x'].copy()
         IS2_atl07_fill[gtx]['sea_ice_segments']['seg_dist_x'] = None
+        IS2_atl07_dims[gtx]['sea_ice_segments']['seg_dist_x'] = ['delta_time']
         IS2_atl07_tide_attrs[gtx]['sea_ice_segments']['seg_dist_x'] = {}
         IS2_atl07_tide_attrs[gtx]['sea_ice_segments']['seg_dist_x']['units'] = "meters"
         IS2_atl07_tide_attrs[gtx]['sea_ice_segments']['seg_dist_x']['contentType'] = "referenceInformation"
@@ -616,6 +681,7 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
         #-- geophysical variables
         IS2_atl07_tide[gtx]['sea_ice_segments']['geophysical'] = {}
         IS2_atl07_fill[gtx]['sea_ice_segments']['geophysical'] = {}
+        IS2_atl07_dims[gtx]['sea_ice_segments']['geophysical'] = {}
         IS2_atl07_tide_attrs[gtx]['sea_ice_segments']['geophysical'] = {}
         IS2_atl07_tide_attrs[gtx]['sea_ice_segments']['geophysical']['Description'] = ("Contains geophysical "
             "parameters and corrections used to correct photon heights for geophysical effects, such as tides.")
@@ -625,6 +691,7 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
         #-- computed tide
         IS2_atl07_tide[gtx]['sea_ice_segments']['geophysical'][variable] = tide.copy()
         IS2_atl07_fill[gtx]['sea_ice_segments']['geophysical'][variable] = tide.fill_value
+        IS2_atl07_dims[gtx]['sea_ice_segments']['geophysical'][variable] = ['delta_time']
         IS2_atl07_tide_attrs[gtx]['sea_ice_segments']['geophysical'][variable] = {}
         IS2_atl07_tide_attrs[gtx]['sea_ice_segments']['geophysical'][variable]['units'] = "meters"
         IS2_atl07_tide_attrs[gtx]['sea_ice_segments']['geophysical'][variable]['contentType'] = "referenceInformation"
@@ -635,20 +702,18 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
         IS2_atl07_tide_attrs[gtx]['sea_ice_segments']['geophysical'][variable]['coordinates'] = \
             "../height_segment_id ../delta_time ../latitude ../longitude"
 
-    #-- output tidal HDF5 file
-    args = (PRD,HEM,TIDE_MODEL,YY,MM,DD,HH,MN,SS,TRK,CYCL,SN,RL,VERS,AUX)
-    ff = '{0}-{1}_{2}_TIDES_{3}{4}{5}{6}{7}{8}_{9}{10}{11}_{12}_{13}{14}.h5'
     #-- print file information
-    print('\t{0}'.format(ff.format(*args))) if VERBOSE else None
-    HDF5_atl07_tide_write(IS2_atl07_tide, IS2_atl07_tide_attrs, CLOBBER=True,
-        INPUT=os.path.basename(FILE), FILL_VALUE=IS2_atl07_fill,
-        FILENAME=os.path.join(DIRECTORY,ff.format(*args)))
+    print('\t{0}'.format(OUTPUT_FILE)) if VERBOSE else None
+    HDF5_ATL07_tide_write(IS2_atl07_tide, IS2_atl07_tide_attrs,
+        CLOBBER=True, INPUT=os.path.basename(INPUT_FILE),
+        FILL_VALUE=IS2_atl07_fill, DIMENSIONS=IS2_atl07_dims,
+        FILENAME=os.path.join(DIRECTORY,OUTPUT_FILE))
     #-- change the permissions mode
-    os.chmod(os.path.join(DIRECTORY,ff.format(*args)), MODE)
+    os.chmod(os.path.join(DIRECTORY,OUTPUT_FILE), MODE)
 
 #-- PURPOSE: outputting the tide values for ICESat-2 data to HDF5
-def HDF5_atl07_tide_write(IS2_atl07_tide, IS2_atl07_attrs, INPUT=None,
-    FILENAME='', FILL_VALUE=None, CLOBBER=False):
+def HDF5_ATL07_tide_write(IS2_atl07_tide, IS2_atl07_attrs, INPUT=None,
+    FILENAME='', FILL_VALUE=None, DIMENSIONS=None, CLOBBER=False):
     #-- setting HDF5 clobber attribute
     if CLOBBER:
         clobber = 'w'
@@ -689,19 +754,8 @@ def HDF5_atl07_tide_write(IS2_atl07_tide, IS2_atl07_attrs, INPUT=None,
             att_val = IS2_atl07_attrs[gtx]['sea_ice_segments'][att_name]
             fileID[gtx]['sea_ice_segments'].attrs[att_name] = att_val
 
-        #-- delta_time
-        v = IS2_atl07_tide[gtx]['sea_ice_segments']['delta_time']
-        attrs = IS2_atl07_attrs[gtx]['sea_ice_segments']['delta_time']
-        #-- Defining the HDF5 dataset variables
-        val = '{0}/{1}/{2}'.format(gtx,'sea_ice_segments','delta_time')
-        h5[gtx]['sea_ice_segments']['delta_time'] = fileID.create_dataset(val,
-            np.shape(v), data=v, dtype=v.dtype, compression='gzip')
-        #-- add HDF5 variable attributes
-        for att_name,att_val in attrs.items():
-            h5[gtx]['sea_ice_segments']['delta_time'].attrs[att_name] = att_val
-
-        #-- geolocation and segment identification variables
-        for k in ['latitude','longitude','height_segment_id',
+        #-- delta_time, geolocation and segment identification variables
+        for k in ['delta_time','latitude','longitude','height_segment_id',
             'geoseg_beg','geoseg_end','seg_dist_x']:
             #-- values and attributes
             v = IS2_atl07_tide[gtx]['sea_ice_segments'][k]
@@ -709,15 +763,22 @@ def HDF5_atl07_tide_write(IS2_atl07_tide, IS2_atl07_attrs, INPUT=None,
             fillvalue = FILL_VALUE[gtx]['sea_ice_segments'][k]
             #-- Defining the HDF5 dataset variables
             val = '{0}/{1}/{2}'.format(gtx,'sea_ice_segments',k)
-            h5[gtx]['sea_ice_segments'][k] = fileID.create_dataset(val,
-                np.shape(v), data=v, dtype=v.dtype, fillvalue=fillvalue,
-                compression='gzip')
-            #-- attach dimensions
-            for dim in ['delta_time']:
-                h5[gtx]['sea_ice_segments'][k].dims.create_scale(
-                    h5[gtx]['sea_ice_segments'][dim], dim)
-                h5[gtx]['sea_ice_segments'][k].dims[0].attach_scale(
-                    h5[gtx]['sea_ice_segments'][dim])
+            if fillvalue:
+                h5[gtx]['sea_ice_segments'][k] = fileID.create_dataset(val,
+                    np.shape(v), data=v, dtype=v.dtype, fillvalue=fillvalue,
+                    compression='gzip')
+            else:
+                h5[gtx]['sea_ice_segments'][k] = fileID.create_dataset(val,
+                    np.shape(v), data=v, dtype=v.dtype, compression='gzip')
+            #-- create or attach dimensions for HDF5 variable
+            if DIMENSIONS[gtx]['sea_ice_segments'][k]:
+                #-- attach dimensions
+                for i,dim in enumerate(DIMENSIONS[gtx]['sea_ice_segments'][k]):
+                    h5[gtx]['sea_ice_segments'][k].dims[i].attach_scale(
+                        h5[gtx]['sea_ice_segments'][dim])
+            else:
+                #-- make dimension
+                h5[gtx]['sea_ice_segments'][k].make_scale(k)
             #-- add HDF5 variable attributes
             for att_name,att_val in attrs.items():
                 h5[gtx]['sea_ice_segments'][k].attrs[att_name] = att_val
@@ -744,10 +805,8 @@ def HDF5_atl07_tide_write(IS2_atl07_tide, IS2_atl07_attrs, INPUT=None,
                     fileID.create_dataset(val, np.shape(v), data=v,
                     dtype=v.dtype, compression='gzip')
             #-- attach dimensions
-            for dim in ['delta_time']:
-                h5[gtx]['sea_ice_segments'][key][k].dims.create_scale(
-                    h5[gtx]['sea_ice_segments'][dim], dim)
-                h5[gtx]['sea_ice_segments'][key][k].dims[0].attach_scale(
+            for i,dim in enumerate(DIMENSIONS[gtx]['sea_ice_segments'][key][k]):
+                h5[gtx]['sea_ice_segments'][key][k].dims[i].attach_scale(
                     h5[gtx]['sea_ice_segments'][dim])
             #-- add HDF5 variable attributes
             for att_name,att_val in attrs.items():
@@ -773,7 +832,7 @@ def HDF5_atl07_tide_write(IS2_atl07_tide, IS2_atl07_attrs, INPUT=None,
     instrument = 'ATLAS > Advanced Topographic Laser Altimeter System'
     fileID.attrs['instrument'] = instrument
     fileID.attrs['source'] = 'Spacecraft'
-    fileID.attrs['references'] = 'http://nsidc.org/data/icesat2/data.html'
+    fileID.attrs['references'] = 'https://nsidc.org/data/icesat-2'
     fileID.attrs['processing_level'] = '4'
     #-- add attributes for input ATL07 file
     fileID.attrs['input_files'] = os.path.basename(INPUT)
@@ -808,14 +867,14 @@ def HDF5_atl07_tide_write(IS2_atl07_tide, IS2_atl07_attrs, INPUT=None,
     #-- convert from seconds since 1980-01-06T00:00:00 to Julian days
     time_julian = 2400000.5 + pyTMD.time.convert_delta_time(gps_seconds - leaps,
         epoch1=(1980,1,6,0,0,0), epoch2=(1858,11,17,0,0,0), scale=1.0/86400.0)
-    #-- convert to calendar date with convert_julian.py
-    YY,MM,DD,HH,MN,SS = convert_julian(time_julian,FORMAT='tuple')
+    #-- convert to calendar date
+    YY,MM,DD,HH,MN,SS = pyTMD.time.convert_julian(time_julian,FORMAT='tuple')
     #-- add attributes with measurement date start, end and duration
-    tcs = datetime.datetime(np.int(YY[0]), np.int(MM[0]), np.int(DD[0]),
-        np.int(HH[0]), np.int(MN[0]), np.int(SS[0]), np.int(1e6*(SS[0] % 1)))
+    tcs = datetime.datetime(int(YY[0]), int(MM[0]), int(DD[0]),
+        int(HH[0]), int(MN[0]), int(SS[0]), int(1e6*(SS[0] % 1)))
     fileID.attrs['time_coverage_start'] = tcs.isoformat()
-    tce = datetime.datetime(np.int(YY[1]), np.int(MM[1]), np.int(DD[1]),
-        np.int(HH[1]), np.int(MN[1]), np.int(SS[1]), np.int(1e6*(SS[1] % 1)))
+    tce = datetime.datetime(int(YY[1]), int(MM[1]), int(DD[1]),
+        int(HH[1]), int(MN[1]), int(SS[1]), int(1e6*(SS[1] % 1)))
     fileID.attrs['time_coverage_end'] = tce.isoformat()
     fileID.attrs['time_coverage_duration'] = '{0:0.0f}'.format(tmx-tmn)
     #-- Closing the HDF5 file
@@ -840,7 +899,7 @@ def main():
         help='Working data directory')
     #-- tide model to use
     model_choices = ('CATS0201','CATS2008','CATS2008_load',
-        'TPXO9-atlas','TPXO9-atlas-v2','TPXO9-atlas-v3',
+        'TPXO9-atlas','TPXO9-atlas-v2','TPXO9-atlas-v3','TPXO9-atlas-v4',
         'TPXO9.1','TPXO8-atlas','TPXO7.2','TPXO7.2_load',
         'AODTM-5','AOTIM-5','AOTIM-5-2018',
         'GOT4.7','GOT4.7_load','GOT4.8','GOT4.8_load','GOT4.10','GOT4.10_load',
@@ -854,6 +913,10 @@ def main():
         metavar='METHOD', type=str, default='spline',
         choices=('spline','linear','nearest','bilinear'),
         help='Spatial interpolation method')
+    #-- extrapolate with nearest-neighbors
+    parser.add_argument('--extrapolate','-E',
+        default=False, action='store_true',
+        help='Extrapolate with nearest-neighbors')
     #-- verbosity settings
     #-- verbose will output information about each output file
     parser.add_argument('--verbose','-V',
@@ -868,7 +931,8 @@ def main():
     #-- run for each input ATL07 file
     for FILE in args.infile:
         compute_tides_ICESat2(args.directory, FILE, TIDE_MODEL=args.tide,
-            METHOD=args.interpolate, VERBOSE=args.verbose, MODE=args.mode)
+            METHOD=args.interpolate, EXTRAPOLATE=args.extrapolate,
+            VERBOSE=args.verbose, MODE=args.mode)
 
 #-- run main program
 if __name__ == '__main__':
